@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
- * USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -34,34 +34,8 @@
 #include "readxml.h"
 #include "rgb.h"
 
-static char * str_extract_until
-  (
-    const char ** src,
-    const char * delim
-  )
-  /* scans *src, looking for the first occurrence of a character in delim. Returns
-    a copy of the prior part of *src if found, and updates *src to point after the
-    delimiter character; else returns a copy of the whole of *src, and sets *src
-    to NULL. Returns NULL iff *src is NULL. */
-  {
-    char * result = NULL; /* to begin with */
-    if (*src != NULL)
-      {
-        const size_t pos = strcspn(*src, delim);
-        if (pos < strlen(*src))
-          {
-            result = strndup(*src, pos);
-            *src = *src + pos + strspn(*src + pos, delim);
-          }
-        else
-          {
-            result = strdup(*src);
-            *src = NULL;
-          } /*if*/
-      } /*if*/
-    return
-        result;
-  } /*str_extract_until*/
+int default_video_format = VF_NONE;
+char provider_str[PROVIDER_SIZE];
 
 /* common parsing bits for both command line and XML file */
 
@@ -77,10 +51,11 @@ static enum
   }
     hadchapter = chapters_neither; /* info about last VOB in current PGC */
 static int
-    pauselen=0,
-    writeoutput=1;
+    pauselen=0;
+static bool
+    writeoutput = true;
 static char
-    *chapters=0;
+    *cur_chapters = 0;
 
 static void parsevideoopts(struct pgcgroup *va,const char *o)
 {
@@ -167,16 +142,15 @@ static void parseinstructions(struct pgc *va,const char *b)
     free(c);
 }
 
-static void parseentries(struct pgc *p,const char *b)
-{
+static void parseentries(struct pgc *p, vtypes ismenuf, const char *b)
+  {
     char *v;
-
-    while(NULL!=(v=str_extract_until(&b,", ")))
+    while (NULL != (v = str_extract_until(&b, ", ")))
       {
-        pgc_add_entry(p,v);
+        pgc_add_entry(p, ismenuf, v);
         free(v);
       } /*while*/
-}
+  } /*parseentries*/
 
 static double parsechapter(const char *s)
 {
@@ -234,7 +208,7 @@ static void readpalette(struct pgc *p,const char *fname)
     struct vfile h;
 
     h=varied_open(fname, O_RDONLY, "palette file");
-    
+
     /* write out colors, the hex is the 0yuv combined in one integer 00yyuuvv */
     i=strlen(fname);
     rgbf=( i>=4 && !strcasecmp(fname+i-4,".rgb") );
@@ -338,7 +312,7 @@ static void usage()
             "\n\t" LONGOPT("--fpc or ") "-F CMD sets the commands to be executed when the disc is first\n"
             "\t    inserted.\n"
             "\n\t" LONGOPT("--jumppad or ") "-j enables the creation of jumppads, which allow greater\n"
-            "\t    flexibility in choosing jump/call desinations.\n"
+            "\t    flexibility in choosing jump/call destinations.\n"
             "\n\t" LONGOPT("--allgprm or ") "-g enables the use of all 16 general purpose registers.\n"
             "\n\t" LONGOPT("--help or ") "-h displays this screen.\n"
         );
@@ -346,46 +320,62 @@ static void usage()
 }
 
 /* bits for authoring via command line */
-#define MAINDEF                                                      \
-            if( !usedtocflag ) {                                     \
-                fprintf(stderr,"ERR:  Must first specify -t, -m, or -x.\n"); \
-                return 1;                                            \
-            }                                                        \
-            if( istoc && curva ) {                                   \
-                fprintf(stderr,"ERR:  TOC cannot have titles\n");    \
-                return 1;                                            \
-            }                                                        \
-            if( !vc ) {                                              \
-                va[curva]=vc=pgcgroup_new(istoc+1-curva);            \
-            }
+#define NOXML                                                                          \
+    if (xmlfile)                                                                       \
+      {                                                                                \
+        fprintf                                                                        \
+          (                                                                            \
+            stderr,                                                                    \
+            "ERR:  Cannot use command line options after specifying XML config file\n" \
+          );                                                                           \
+        return 1;                                                                      \
+      } /*if*/
+#define MAINDEF                                                               \
+            if (!usedtocflag)                                                 \
+              {                                                               \
+                fprintf(stderr, "ERR:  Must first specify -t, -m, or -x.\n"); \
+                return 1;                                                     \
+              } /*if*/                                                        \
+            if (istoc && istitle)                                             \
+              {                                                               \
+                fprintf(stderr, "ERR:  TOC cannot have titles\n");            \
+                return 1;                                                     \
+              } /*if*/                                                        \
+            if (!va[istitle])                                                          \
+              {                                                               \
+                va[istitle] = pgcgroup_new((int)istoc + 1 - (int)istitle); \
+              } /*if*/
 
 #define MAINDEFPGC                                                   \
-            MAINDEF;                                                 \
-            if( !curpgc )                                            \
-                curpgc=pgc_new();
+            MAINDEF                                                  \
+            if (!curpgc)                                             \
+                curpgc = pgc_new();
 
 #define MAINDEFVOB                                                   \
-            MAINDEFPGC;                                              \
-            if( !curvob )                                            \
-                curvob=source_new();
+            MAINDEFPGC                                               \
+            if (!curvob)                                             \
+                curvob = source_new();
 
 #define FLUSHPGC                                                     \
-            if( curpgc ) {                                           \
-                pgcgroup_add_pgc(va[curva],curpgc);                   \
-                curpgc=0;                                            \
-            }
+            if (curpgc)                                              \
+              {                                                      \
+                pgcgroup_add_pgc(va[istitle], curpgc);               \
+                curpgc = 0;                                          \
+              } /*if*/
 
-int main(int argc,char **argv)
-{
-    struct pgcgroup *va[2];
+int main(int argc, char **argv)
+  {
+    struct pgcgroup *va[2]; /* element 0 for doing menus, 1 for doing titles */
     struct menugroup *mg;
-    char *fbase=0; /* output directory name */
-    int curva=1,istoc=0,l,
-        usedtocflag=0; // whether the 'istoc' value has been used or not
-    struct pgc *curpgc=0,*fpc=0;
-    struct source *curvob=0;
+    char *fbase = 0; /* output directory name */
+    char * xmlfile = 0; /* name of XML control file, if any */
+    bool istitle = true; /* index into va */
+    bool istoc = false, /* true if doing VMG, false if doing titleset */
+        usedtocflag = false; /* indicates that istoc can no longer be changed */
+    struct pgc *curpgc = 0,* fpc = 0;
+    struct source *curvob = 0;
 #ifdef HAVE_GETOPT_LONG
-    static struct option longopts[]={
+    const static struct option longopts[]={
         {"video",1,0,'v'},
         {"audio",1,0,'a'},
         {"subpictures",1,0,'s'},
@@ -409,33 +399,69 @@ int main(int argc,char **argv)
 #define GETOPTFUNC(x,y,z) getopt(x,y,z)
 #endif
 
-    fputs(PACKAGE_HEADER("dvdauthor"),stderr);
+    default_video_format = get_video_format();
+    init_locale();
+    fputs(PACKAGE_HEADER("dvdauthor"), stderr);
+    if (default_video_format != VF_NONE)
+      {
+        fprintf
+          (
+            stderr,
+            "INFO: default video format is %s\n",
+            default_video_format == VF_PAL ? "PAL" : "NTSC"
+          );
+      }
+    else
+      {
+#if defined(DEFAULT_VIDEO_FORMAT)
+#    if DEFAULT_VIDEO_FORMAT == 1
+        fprintf(stderr, "INFO: default video format is NTSC\n");
+#    elif DEFAULT_VIDEO_FORMAT == 2
+        fprintf(stderr, "INFO: default video format is PAL\n");
+#    endif
+#else
+        fprintf(stderr, "INFO: no default video format, must explicitly specify NTSC or PAL\n");
+#endif
+      } /*if*/
 
-    if( argc<1 ) {
-        fprintf(stderr,"ERR:  No arguments!\n");
+    if (argc < 1)
+      {
+        fprintf(stderr, "ERR:  No arguments!\n");
         return 1;
-    }
-    memset(va,0,sizeof(struct pgcgroup *)*2);
+      } /*if*/
+    memset(va, 0, sizeof(struct pgcgroup *) * 2);
 
-    while(1) {
-        struct pgcgroup *vc=va[curva];
-        int c=GETOPTFUNC(argc,argv,"f:o:v:a:s:hc:Cp:Pmtb:Ti:e:x:jgn");
-        if( c == -1 )
+    while (true)
+      {
+        int c = GETOPTFUNC(argc, argv, "f:o:O:v:a:s:hc:Cp:Pmtb:Ti:e:x:jgn");
+        if (c == -1)
             break;
-        switch(c) {
+        switch (c)
+          {
         case 'h':
             usage();
         break;
 
         case 'x':
-            if( usedtocflag ) {
-                fprintf(stderr,"ERR:  Cannot specify XML config file after using command line options\n");
+            if (usedtocflag)
+              {
+                fprintf
+                  (
+                    stderr,
+                    "ERR:  Cannot specify XML config file after using command line options\n"
+                  );
                 return 1;
-            }
-            return readdvdauthorxml(optarg,fbase);
+              } /*if*/
+            if (xmlfile)
+              {
+                fprintf(stderr, "ERR:  only one XML file allowed\n");
+                return 1;
+              } /*if*/
+            xmlfile = optarg;
+        break;
 
         case 'n':
-            writeoutput=0;
+            writeoutput = false;
         break;
 
         case 'j':
@@ -447,180 +473,217 @@ int main(int argc,char **argv)
         break;
 
         case 'T':
-            if( usedtocflag ) {
-                fprintf(stderr,"ERR:  TOC (-T) option must come first because I am a lazy programmer.\n");
+            NOXML
+            if (usedtocflag)
+              {
+                fprintf
+                  (
+                    stderr,
+                    "ERR:  TOC (-T) option must come first because I am a lazy programmer.\n"
+                  );
                 return 1;
-            }
-            istoc=1;
-            usedtocflag=1; // just for completeness (also see -x FOO)
+              } /*if*/
+            istoc = true;
+            usedtocflag = true; // just for completeness (also see -x FOO)
         break;
 
-        case 'o': 
-            fbase=optarg;
+        case 'O':
+            delete_output_dir = true;
+        /* and fallthru */
+        case 'o':
+            fbase = optarg;
         break;
 
         case 'm':
-            FLUSHPGC;
-            usedtocflag=1; // force -T to occur before -m
-            hadchapter=chapters_neither; /* reset for new menu */
-            curva=0;
+            NOXML
+            FLUSHPGC
+            usedtocflag = true; // force -T to occur before -m
+            hadchapter = chapters_neither; /* reset for new menu */
+            istitle = false;
         break;
 
         case 't':
-            if( istoc ) {
-                fprintf(stderr,"ERR:  TOC cannot have titles\n");
+            NOXML
+            if (istoc)
+              {
+                fprintf(stderr, "ERR:  TOC cannot have titles\n");
                 return 1;
-            }
-            FLUSHPGC;
-            usedtocflag=1;
-            hadchapter=chapters_neither; /* reset for new title */
-            curva=1;
+              } /*if*/
+            FLUSHPGC
+            usedtocflag = true;
+            hadchapter = chapters_neither; /* reset for new title */
+            istitle = true;
         break;
-            
+
         case 'a':
-            MAINDEF;
-            parseaudioopts(vc,optarg);
+            NOXML
+            MAINDEF
+            parseaudioopts(va[istitle], optarg);
         break;
-        
+
         case 'v':
-            MAINDEF;
-            parsevideoopts(vc,optarg);
+            NOXML
+            MAINDEF
+            parsevideoopts(va[istitle], optarg);
         break;
 
         case 's':
-            MAINDEF;
-            parsesubpictureopts(vc,optarg);
+            NOXML
+            MAINDEF
+            parsesubpictureopts(va[istitle], optarg);
         break;
 
         case 'b':
-            MAINDEFPGC;
-            parsebutton(curpgc,optarg);
+            NOXML
+            MAINDEFPGC
+            parsebutton(curpgc, optarg);
         break;
 
         case 'i':
-            MAINDEFPGC;
-            parseinstructions(curpgc,optarg);
+            NOXML
+            MAINDEFPGC
+            parseinstructions(curpgc, optarg);
         break;
 
         case 'F':
-            if( !istoc ) {
-                fprintf(stderr,"ERR:  You may only specify FPC commands on the VMGM\n");
+            NOXML
+            if (!istoc)
+              {
+                fprintf(stderr, "ERR:  You may only specify FPC commands on the VMGM\n");
                 return 1;
-            }
-            usedtocflag=1;
-            if( fpc ) {
+              } /*if*/
+            usedtocflag = true;
+            if (fpc)
+              {
                 fprintf(stderr,"ERR:  FPC commands already specified\n");
                 return 1;
-            }
-            fpc=pgc_new();
-            pgc_set_pre(fpc,optarg);
+              } /*if*/
+            fpc = pgc_new();
+            pgc_set_pre(fpc, optarg);
         break;
 
         case 'e':
-            MAINDEFPGC;
-            if( curva ) {
-                fprintf(stderr,"ERR:  Cannot specify an entry for a title.\n");
+            NOXML
+            MAINDEFPGC
+            if (istitle)
+              {
+                fprintf(stderr, "ERR:  Cannot specify an entry for a title.\n");
                 return 1;
-            }
-            parseentries(curpgc,optarg);
+              } /*if*/
+            parseentries(curpgc, istoc ? VTYPE_VMGM : VTYPE_VTSM, optarg);
         break;
 
         case 'P':
-            optarg=0;
+          /* NOXML */
+            optarg = 0; /* use default palette name */
       /* fallthru */
         case 'p':
-            MAINDEFPGC;
-            readpalette(curpgc,optarg?optarg:"xste-palette.dat");
+            NOXML
+            MAINDEFPGC
+            readpalette(curpgc, optarg ? optarg : "xste-palette.dat");
         break;
 
         case 1:
         case 'f':
-            MAINDEFVOB;
-            
-            source_set_filename(curvob,optarg);
-            if( hadchapter==chapters_chapters )
-                hadchapter=chapters_cells; /* chapters already specified */
+            NOXML
+            MAINDEFVOB
+            source_set_filename(curvob, optarg);
+            if (hadchapter == chapters_chapters )
+                hadchapter = chapters_cells; /* chapters already specified */
             else
                 source_add_cell
                   (
                     /*source =*/ curvob,
                     /*starttime =*/ 0,
                     /*endtime =*/ -1,
-                    /*chap =*/ hadchapter == chapters_neither ? CELL_CHAPTER_PROGRAM : CELL_NEITHER,
+                    /*chap =*/
+                        hadchapter == chapters_neither ?
+                            CELL_CHAPTER_PROGRAM /* first vob in pgc */
+                        :
+                            CELL_NEITHER, /* subsequent vobs in pgc */
                     /*pause =*/ 0,
                     /*cmd =*/ 0
                   );
-                  /* default to single chapter for entire source file */
-            pgc_add_source(curpgc,curvob);
-            curvob=0;
+                  /* default to single chapter/cell for entire source file */
+            pgc_add_source(curpgc, curvob);
+            curvob = 0;
         break;
 
         case 'C':
-            optarg=0;
+          /* NOXML */
+            optarg = 0;
       /* fallthru */
         case 'c':
-            if( curvob ) {
-                fprintf(stderr,"ERR:  cannot list -c twice for one file.\n");
+            NOXML
+            if (curvob)
+              {
+                fprintf(stderr, "ERR:  cannot list -c twice for one file.\n");
                 return 1;
-            }
-            MAINDEFVOB;
-
-            hadchapter=chapters_chapters;
-            if( optarg )
-                parsechapters(optarg,curvob,0);
+              } /*if*/
+            MAINDEFVOB
+            hadchapter = chapters_chapters;
+            if (optarg)
+                parsechapters(optarg, curvob, 0);
             else
                 source_add_cell(curvob, 0, -1, CELL_CHAPTER_PROGRAM, 0, 0);
                   /* default to single chapter for entire source file */
         break;
 
         default:
-            fprintf(stderr,"ERR:  getopt returned bad code %d\n",c);
+            fprintf(stderr, "ERR:  getopt returned bad code %d\n", c);
             return 1;
-        }
-    }
-    if( curvob ) {
-        fprintf(stderr,"ERR:  Chapters defined without a file source.\n");
-        return 1;
-    }
-    FLUSHPGC;
-    if( !va[0] ) {
-        va[0]=pgcgroup_new(istoc+1);
-        mg=menugroup_new();
-        menugroup_add_pgcgroup(mg,"en",va[0]);
-    } else
-        mg=0;
-    if( !va[1] && !istoc )
-        va[1]=pgcgroup_new(VTYPE_VTS);
-
-    if( !fbase && writeoutput ) {
-        fbase=readconfentry("WORKDIR");
-        if( !fbase )
-            usage();
-    }
-    if( optind != argc ) {
-        fprintf(stderr,"ERR:  bad version of getopt; please precede all sources with '-f'\n");
-        return 1;
-    }
-    if( fbase ) {
-        l=strlen(fbase);
-        if( l && fbase[l-1]=='/' )
-            fbase[l-1]=0;
-    }
-
-    if( istoc )
-        dvdauthor_vmgm_gen(fpc,mg,fbase);
+          } /*switch*/
+      } /*while*/
+    if (xmlfile)
+      {
+        return readdvdauthorxml(xmlfile, fbase);
+      }
     else
-        dvdauthor_vts_gen(mg,va[1],fbase);
-
-    if( fpc )
+      {
+        if (curvob)
+          {
+            fprintf(stderr, "ERR:  Chapters defined without a file source.\n");
+            return 1;
+          } /*if*/
+        FLUSHPGC
+        if (!va[0])
+          {
+            va[0] = pgcgroup_new(istoc ? VTYPE_VMGM : VTYPE_VTSM);
+            mg = menugroup_new();
+            menugroup_add_pgcgroup(mg, "en", va[0]);
+              /* fixme: would be nice to make language a preference setting, perhaps default to locale? */
+          }
+        else
+            mg = 0;
+        if (!va[1] && !istoc)
+            va[1] = pgcgroup_new(VTYPE_VTS);
+        if (!fbase && writeoutput)
+          {
+            fbase = get_outputdir();
+            if (!fbase)
+                usage();
+          } /*if*/
+        if (optind != argc)
+          {
+            fprintf(stderr, "ERR:  bad version of getopt; please precede all sources with '-f'\n");
+            return 1;
+          } /*if*/
+        if (fbase)
+          {
+            const int l = strlen(fbase);
+            if (l && fbase[l - 1] == '/')
+                fbase[l - 1] = 0;
+          } /*if*/
+        if (istoc)
+            dvdauthor_vmgm_gen(fpc, mg, fbase);
+        else
+            dvdauthor_vts_gen(mg, va[1], fbase);
         pgc_free(fpc);
-    if( mg )
         menugroup_free(mg);
-    if( va[1] )
         pgcgroup_free(va[1]);
-
-    return 0;
-}
+        return 0;
+      } /*if*/
+  } /*main*/
 
 /* authoring via XML file */
 
@@ -644,12 +707,14 @@ static struct menugroup
 static struct pgc *curpgc=0,*fpc=0;
 static struct source *curvob=0;
 static const char
-    *fbase=0, /* output directory name */
-    *buttonname=0; /* name of button currently being defined */
+    *fbase = 0, /* output directory name */
+    *buttonname = 0; /* name of button currently being defined */
 static vtypes
-    ismenuf = VTYPE_VTS;
+    ismenuf = VTYPE_VTS; /* type of current pgcgroup structure being parsed */
+static bool
+    istoc = false, /* true for vmgm, false for titleset */
+    hadtoc = false; /* set to true when vmgm seen */
 static int
-    istoc=0, /* 1 for vmgm, 0 for titleset */
     setvideo=0, /* to keep count of <video> tags */
     setaudio=0, /* to keep count of <audio> tags */
     setsubpicture=0, /* to keep count of <subpicture> tags */
@@ -659,7 +724,6 @@ static int
         DA_PGC -- inside a <pgc> tag (no "lang" attribute allowed)
         DA_PGCGROUP -- inside a <menus> or <titles> tag, must come before the <pgc> tags
       */
-    hadtoc=0, /* set to 1 when vmgm seen */
     subpstreamid=-1; /* id attribute of current <stream> saved here */
 static char *subpstreammode=0; /* mode attribute of current <stream> saved here */
 static enum
@@ -717,93 +781,118 @@ static int parse_pause(const char *f)
         return strtounsigned(f, "pause time"); /* should check it's in [0 .. 254] */
 }
 
-static void dvdauthor_workdir(const char *s)
-{
-    fbase=utf8tolocal(s);
-}
+static void dvdauthor_outputdir(const char *s)
+  {
+    if (!fbase)
+      {
+        fbase = localize_filename(s);
+      } /*if*/
+  }
 
 static void dvdauthor_jumppad(const char *s)
 {
-    int i=xml_ison(s);
-    if( i==1 )
+    if(xml_ison(s, "jumppad"))
         dvdauthor_enable_jumppad();
-    else if( i==-1 ) {
-        fprintf(stderr,"ERR:  Unknown jumppad cmd '%s'\n",s);
-        exit(1);
-    }
 }
 
 static void dvdauthor_allgprm(const char *s)
 {
-    int i=xml_ison(s);
-    if( i==1 )
+    if (xml_ison(s, "allgprm"))
         dvdauthor_enable_allgprm();
-    else if( i==-1 ) {
-        fprintf(stderr,"ERR:  Unknown allgprm cmd '%s'\n",s);
-        exit(1);
-    }
 }
 
+static void dvdauthor_video_format
+  (
+    const char * s
+  )
+  {
+    if (!strcmp(s, "ntsc") || !strcmp(s, "NTSC"))
+      {
+        default_video_format = VF_NTSC;
+      }
+    else if (!strcmp(s, "pal") || !strcmp(s, "PAL"))
+      {
+        default_video_format = VF_PAL;
+      }
+    else
+      {
+        fprintf(stderr, "ERR:  Unrecognized video format \"%s\"\n", s);
+        parser_err = true;
+      } /*if*/
+  } /*dvdauthor_video_format*/
+
+static void dvdauthor_provider
+  (
+    const char * s
+  )
+  {
+    strncpy(provider_str, s, PROVIDER_SIZE);
+    provider_str[PROVIDER_SIZE - 1] = 0;
+  } /*dvdauthor_provider*/
+
 static void getfbase()
-{
-    if( !writeoutput )
-        fbase=0;
-    else if( !fbase ) {
-        fbase=readconfentry("WORKDIR");
-        if( !fbase ) {
-            fprintf(stderr,"ERR:  Must specify working directory\n");
-            parser_err=1;
-        }
-    }
-}
+  {
+    if (!writeoutput)
+        fbase = 0;
+    else if (!fbase)
+      {
+        fprintf(stderr, "ERR:  Must specify working directory\n");
+        parser_err = true;
+      } /*if*/
+  } /*getfbase*/
 
-static void dvdauthor_end()
+static void dvdauthor_start(void)
+  {
+    strncpy(provider_str, PACKAGE_STRING, PROVIDER_SIZE);
+    provider_str[PROVIDER_SIZE - 1] = 0;
+  } /*dvdauthor_start*/
+
+static void dvdauthor_end(void)
 /* called on </dvdauthor> end tag, generates the VMGM if specified.
   This needs to be done after all the titles, so it can include
   information about them. */
-{
-    if( hadtoc ) {
-        dvdauthor_vmgm_gen(fpc,vmgmmenus,fbase);
-        if( fpc )
-            pgc_free(fpc);
-        if( vmgmmenus )
-            menugroup_free(vmgmmenus);
-        fpc=0;
-        vmgmmenus=0;
-    }
-}
+  {
+    if (hadtoc)
+      {
+        dvdauthor_vmgm_gen(fpc, vmgmmenus, fbase);
+        pgc_free(fpc);
+        menugroup_free(vmgmmenus);
+        fpc = 0;
+        vmgmmenus = 0;
+      } /*if*/
+  } /*dvdauthor_end*/
 
 static void titleset_start()
 {
     mg=menugroup_new();
-    istoc=0;
+    istoc = false;
 }
 
 static void titleset_end()
-{ /* called on </titles> end tag, generates the output titleset. */
+  { /* called on </titles> end tag, generates the output titleset. */
     getfbase();
-    if( !parser_err ) {
+    if (!parser_err)
+      {
         if (!titles)
-            titles=pgcgroup_new(VTYPE_VTS);
-        dvdauthor_vts_gen(mg,titles,fbase);
-        if (mg)
-            menugroup_free(mg);
+            titles = pgcgroup_new(VTYPE_VTS);
+        dvdauthor_vts_gen(mg, titles, fbase);
+        menugroup_free(mg);
         pgcgroup_free(titles);
-        mg=0;
-        titles=0;
-    }
-}
+        mg = 0;
+        titles = 0;
+      } /*if*/
+  } /*titleset_end*/
 
 static void vmgm_start()
 {
     if (hadtoc) {
         fprintf(stderr,"ERR:  Can only define one VMGM\n");
-        parser_err=1;
+        parser_err = true;
         return;
     }
     mg=menugroup_new();
-    istoc=1;
-    hadtoc=1;
+    istoc = true;
+    hadtoc = true;
 }
 
 static void vmgm_end()
@@ -818,16 +907,16 @@ static void fpc_start()
 {
     if( !istoc ) {
         fprintf(stderr,"ERR:  Can only specify <fpc> under <vmgm>\n");
-        parser_err=1;
+        parser_err = true;
         return;
     }
     if( fpc ) {
         fprintf(stderr,"ERR:  Already defined <fpc>\n");
-        parser_err=1;
+        parser_err = true;
         return;
     }
     fpc=pgc_new();
-    parser_acceptbody=1;
+    parser_acceptbody = true;
 }
 
 static void fpc_end()
@@ -850,12 +939,12 @@ static void titles_start()
     if (titles)
       {
         fprintf(stderr, "ERR:  Titles already defined\n");
-        parser_err = 1;
+        parser_err = true;
       }
     else if (istoc)
       {
         fprintf(stderr, "ERR:  Cannot have titles in a VMGM\n");
-        parser_err = 1;
+        parser_err = true;
       }
     else
       {
@@ -896,7 +985,7 @@ static void video_start()
       {
         fprintf(stderr, "ERR:  Already defined video characteristics for this PGC group\n");
           /* only one video stream allowed */
-        parser_err = 1;
+        parser_err = true;
       }
     else
         setvideo = 1;
@@ -934,7 +1023,7 @@ static void audio_start()
     if (setaudio >= 8)
       {
         fprintf(stderr, "ERR:  Attempting to define too many audio streams for this PGC group\n");
-        parser_err = 1;
+        parser_err = true;
       } /*if*/
   }
 
@@ -979,7 +1068,7 @@ static void audio_channels(const char *c)
         ch[3] = 0;
         c = ch;
       } /*if*/
-    set_audio_attr(AUDIO_CHANNELS, ch, setaudio);
+    set_audio_attr(AUDIO_CHANNELS, c, setaudio);
   }
 
 static void subattr_group_start()
@@ -988,14 +1077,14 @@ static void subattr_group_start()
     if (subpmode != DA_PGCGROUP)
       {
         fprintf(stderr, "ERR:  Define all the subpictures before defining PGCs\n");
-        parser_err = 1;
+        parser_err = true;
         return;
       } /*if*/
     setsubpicture++;
     if (setsubpicture >= 32)
       {
         fprintf(stderr, "ERR:  Attempting to define too many subpicture streams for this PGC group\n");
-        parser_err = 1;
+        parser_err = true;
       } /*if*/
   } /*subattr_group_start*/
 
@@ -1006,7 +1095,7 @@ static void subattr_pgc_start()
     if (setsubpicture >= 32)
       {
         fprintf(stderr, "ERR:  Attempting to define too many subpicture streams for this PGC group\n");
-        parser_err = 1;
+        parser_err = true;
       } /*if*/
   } /*subattr_pgc_start*/
 
@@ -1021,7 +1110,7 @@ static void subattr_lang(const char *c)
             stderr,
             "ERR:  Cannot set subpicture language within a pgc; do it within titles or menus\n"
           );
-        parser_err = 1;
+        parser_err = true;
       } /*if*/
   } /*subattr_lang*/
 
@@ -1038,7 +1127,7 @@ static void subattr_content(const char * c)
             stderr,
             "ERR:  Cannot set subpicture content type within a pgc; do it within titles or menus\n"
           );
-        parser_err = 1;
+        parser_err = true;
       } /*if*/
   } /*subattr_content*/
 
@@ -1056,7 +1145,7 @@ static void substream_id(const char *c)
     if (subpstreamid < 0 || subpstreamid >= 32)
       {
         fprintf(stderr, "ERR:  Subpicture stream id must be 0-31: '%s'.\n", c);
-        parser_err = 1;
+        parser_err = true;
       } /*if*/
   } /*substream_id*/
 
@@ -1071,7 +1160,7 @@ static void stream_end()
     if (subpstreamid == -1 || !subpstreammode)
       {
         fprintf(stderr, "ERR:  Must define the mode and id for the stream.\n");
-        parser_err = 1;
+        parser_err = true;
       }
     else if (subpmode == DA_PGCGROUP)
       {
@@ -1094,7 +1183,7 @@ static void pgc_entry(const char *e)
 {
     // xml attributes can only be defined once, so entry="foo" entry="bar" won't work
     // instead, use parseentries...
-    parseentries(curpgc, e);
+    parseentries(curpgc, ismenuf, e);
 }
 
 static void pgc_palette(const char *p)
@@ -1116,7 +1205,7 @@ static void pgc_end()
 
 static void pre_start()
 {
-    parser_acceptbody=1;
+    parser_acceptbody = true;
 }
 
 static void pre_end()
@@ -1126,7 +1215,7 @@ static void pre_end()
 
 static void post_start()
 {
-    parser_acceptbody=1;
+    parser_acceptbody = true;
 }
 
 static void post_end()
@@ -1146,15 +1235,16 @@ static void vob_start()
 
 static void vob_file(const char *f)
 {
-    f = utf8tolocal(f);
+    f = localize_filename(f);
     source_set_filename(curvob, f);
+    free(f);
 }
 
 static void vob_chapters(const char *c)
   {
     vobbasic = vob_has_chapters_pause;
     hadchapter = chapters_chapters;
-    chapters = strdup(c); /* fixme: leaks! */
+    cur_chapters = strdup(c); /* won't leak, because I won't be called more than once per <vob> */
   }
 
 static void vob_pause(const char *c)
@@ -1169,31 +1259,36 @@ static void vob_end()
       {
         if (hadchapter == chapters_chapters) /* vob has chapters attribute */
           {
-          /* fixme: hadchapter could still be set to this value from previous vob,
-            in which case I wrongly apply same chapter definitions to this vob! */
-            parsechapters(chapters, curvob, pauselen);
+            parsechapters(cur_chapters, curvob, pauselen);
             hadchapter = chapters_cells;
+              /* subsequent <vob>s in this PGC shall be only cells, unless
+                they have their own "chapter" attribute */
           }
         else /* hadchapter = chapters_neither or chapters_cells */
-          /* I think this should only ever be chapters_neither */
             source_add_cell
               (
                 /*source =*/ curvob,
                 /*starttime =*/ 0,
                 /*endtime =*/ -1,
-                /*chap =*/ hadchapter == chapters_neither ? CELL_CHAPTER_PROGRAM : CELL_NEITHER,
+                /*chap =*/
+                    hadchapter == chapters_neither ?
+                        CELL_CHAPTER_PROGRAM /* first <vob> in <pgc> */
+                    :
+                        CELL_NEITHER, /* subsequent <vob>s in <pgc> */
                 /*pause =*/ pauselen,
                 /*cmd =*/ 0
               );
-              /* default to single chapter for entire source file */
+              /* default to single chapter/cell for entire source file */
       } /*if*/
     pgc_add_source(curpgc, curvob);
+    free(cur_chapters);
+    cur_chapters = 0;
     curvob = 0;
   } /*vob_end*/
 
 static void cell_start()
   {
-    parser_acceptbody = 1; /* collect cell commands */
+    parser_acceptbody = true; /* collect cell commands */
     assert(vobbasic != vob_has_chapters_pause);
       /* no "chapters" or "pause" attribute on containing <vob> allowed */
     vobbasic = vob_has_cells;
@@ -1206,35 +1301,33 @@ static void cell_start()
 
 static void cell_parsestart(const char *f)
   {
+    if (f[0] == 0)
+      {
+        fprintf(stderr,"ERR:  Empty cell start time\n");
+        exit(1);
+      } /*if*/
     cell_starttime = parsechapter(f);
   }
 
 static void cell_parseend(const char *f)
   {
+    if (f[0] == 0)
+      {
+        fprintf(stderr,"ERR:  Empty cell end time\n");
+        exit(1);
+      } /*if*/
     cell_endtime = parsechapter(f);
   }
 
 static void cell_parsechapter(const char *f)
   {
-    const int i = xml_ison(f);
-    if (i == -1)
-      {
-        fprintf(stderr, "ERR:  Unknown chapter cmd '%s'\n", f);
-        exit(1);
-      }
-    else if (i)
+    if (xml_ison(f, "chapter"))
         cell_chapter = CELL_CHAPTER_PROGRAM;
   }
 
 static void cell_parseprogram(const char *f)
   {
-    const int i = xml_ison(f);
-    if (i == -1)
-      {
-        fprintf(stderr, "ERR:  Unknown program cmd '%s'\n", f);
-        exit(1);
-      }
-    else if (i && cell_chapter != CELL_CHAPTER_PROGRAM)
+    if (xml_ison(f, "program") && cell_chapter != CELL_CHAPTER_PROGRAM)
         cell_chapter = CELL_PROGRAM;
   }
 
@@ -1252,7 +1345,7 @@ static void cell_end()
 
 static void button_start()
 {
-    parser_acceptbody=1;
+    parser_acceptbody = true;
     buttonname=0;
 }
 
@@ -1268,7 +1361,7 @@ static void button_end()
 }
 
 static struct elemdesc elems[]={
-    {"dvdauthor", DA_BEGIN,   DA_ROOT,    0,               dvdauthor_end},
+    {"dvdauthor", DA_BEGIN,   DA_ROOT,    dvdauthor_start, dvdauthor_end},
     {"titleset",  DA_ROOT,    DA_SET,     titleset_start,  titleset_end},
     {"vmgm",      DA_ROOT,    DA_SET,     vmgm_start,      vmgm_end},
     {"fpc",       DA_SET,     DA_NOSUB,   fpc_start,       fpc_end},
@@ -1289,9 +1382,11 @@ static struct elemdesc elems[]={
 };
 
 static struct elemattr attrs[]={
-    {"dvdauthor","dest",dvdauthor_workdir},
+    {"dvdauthor","dest",dvdauthor_outputdir},
     {"dvdauthor","jumppad",dvdauthor_jumppad},
     {"dvdauthor","allgprm",dvdauthor_allgprm},
+    {"dvdauthor","format",dvdauthor_video_format},
+    {"dvdauthor","provider",dvdauthor_provider},
 
     {"menus","lang",menus_lang},
 
@@ -1332,8 +1427,12 @@ static struct elemattr attrs[]={
     {0,0,0}
 };
 
-static int readdvdauthorxml(const char *xmlfile,const char *fb)
+static int readdvdauthorxml(const char *xmlfile, const char *fb)
 {
-    fbase=fb;
-    return readxml(xmlfile,elems,attrs);
+    fbase = fb;
+    if (!fbase)
+      {
+        fbase = get_outputdir();
+      } /*if*/
+    return readxml(xmlfile, elems, attrs);
 }
