@@ -16,14 +16,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
- * USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301 USA.
  */
 
 #include "config.h"
-
 #include "compat.h"
-
+#include <errno.h>
 #include <assert.h>
 
 #include "dvdauthor.h"
@@ -34,6 +33,9 @@ static unsigned char *
 static size_t
     bigbufsize = 0;
 
+#define MIN_IFO_SECTORS 16
+  /* need to ensure that .IFO and .BUP files end up in different ECC blocks */
+
 static void buf_init()
   /* ensures there's no leftover junk in bigbuf. */
   {
@@ -114,9 +116,48 @@ static void nfwrite(const void *ptr, size_t len, FILE *h)
   /* writes to h, or turns into a noop if h is null. */
   {
     if (h)
-        fwrite(ptr, len, 1, h);
+      {
+        errno = 0; /* where is this being set? */
+        (void)fwrite(ptr, len, 1, h);
+        if (errno != 0)
+          {
+            fprintf
+              (
+                stderr,
+                "\nERR:  Error %d -- %s -- writing output IFO\n",
+                errno,
+                strerror(errno)
+              );
+            exit(1);
+          } /*if*/
+      } /*if*/
   } /*nfwrite*/
 
+static void nfpad(size_t len, FILE *h)
+  /* writes len bytes of padding to h, or turns into a noop if h is null. */
+  {
+    static unsigned char buf[2048];
+    static size_t himark = 0;
+    if (h != NULL)
+      {
+        while (len != 0)
+          {
+            size_t uselen = sizeof buf;
+            if (uselen > len)
+              {
+                uselen = len;
+              } /*if*/
+            if (uselen > himark)
+              {
+                memset(buf + himark, 0, uselen - himark);
+                himark = uselen;
+              } /*if*/
+            nfwrite(buf, uselen, h);
+            len -= uselen;
+          } /*while*/
+      } /*if*/
+  } /*nfpad*/
+
 static const struct vobuinfo *globalfindvobu(const struct pgc *ch, int pts)
   /* finds the VOBU spanning the specified time. */
   {
@@ -191,7 +232,7 @@ static int get_pgc_duration_seconds(const struct pgcgroup *va, int c)
     // an exact multiple of 90090*units; if so, then the last entry of the
     // TMAPT table cannot be properly computed, because that entry will have
     // fallen off the end of the VOBU table
-    return pts_ticks_to_seconds(va->vg, getptsspan(va->pgcs[c]) - 1);
+    return pts_ticks_to_seconds(va->pg_vg, getptsspan(va->pgcs[c]) - 1);
   } /*get_pgc_duration_seconds*/
 
 static int secunit(int ns)
@@ -268,15 +309,15 @@ static void CreateTMAPT(FILE *h, const struct pgcgroup *va)
           /* write VTS_TMAP entries */
             const struct vobuinfo *vobu1;
             // I don't know why I ever did this
-            // ptsbase = -getframepts(va->vg);
+            // ptsbase = -getframepts(va->pg_vg);
             ptsbase = 0; // this matches Bullitt
-            vobu1 = globalfindvobu(thispgc, ptsbase + pts_seconds_to_ticks(va->vg, units));
+            vobu1 = globalfindvobu(thispgc, ptsbase + pts_seconds_to_ticks(va->pg_vg, units));
             for (j = 0; j < numtmapt; j++)
               {
                 const struct vobuinfo * const vobu2 = globalfindvobu
                   (
                     thispgc,
-                    ptsbase + pts_seconds_to_ticks(va->vg, (j + 2) * units)
+                    ptsbase + pts_seconds_to_ticks(va->pg_vg, (j + 2) * units)
                   );
                 write4(buf, vobu1->sector);
                 if (!vobu2 || vobu1->vobcellid != vobu2->vobcellid)
@@ -322,11 +363,11 @@ static int CreateCellAddressTable(FILE *h, const struct vobgroup *va)
         const struct vob * const thisvob = va->vobs[k];
         for (i = 0; i < thisvob->numvobus; i++)
           {
-            if (!i || thisvob->vobu[i].vobcellid != thisvob->vobu[i-1].vobcellid)
+            if (!i || thisvob->vobu[i].vobcellid != thisvob->vobu[i - 1].vobcellid)
               { /* starting a new cell */
                 if (i)
                   {
-                    buf_write4(p + 8, thisvob->vobu[i-1].lastsector);
+                    buf_write4(p + 8, thisvob->vobu[i - 1].lastsector);
                       /* ending sector within VOB in previous entry */
                     p += 12;
                   } /*if*/
@@ -335,7 +376,7 @@ static int CreateCellAddressTable(FILE *h, const struct vobgroup *va)
                 buf_write4(p + 4, thisvob->vobu[i].sector); /* starting sector within VOB */
               } /*if*/
           } /*for*/
-        buf_write4(p + 8, thisvob->vobu[i-1].lastsector);
+        buf_write4(p + 8, thisvob->vobu[i - 1].lastsector);
           /* ending sector within VOB in last entry */
         p += 12;
       } /*for*/
@@ -410,6 +451,10 @@ static int Create_PTT_SRPT(FILE *h, const struct pgcgroup *t)
                   /* fallthru */
                     case CELL_PROGRAM:
                         pgm++; /* keep right count for next chapter */
+                    break;
+                    case CELL_NEITHER:
+                      /* fine */
+                    break;
                       } /*switch*/
               } /*for; for*/
       } /*for*/
@@ -530,13 +575,13 @@ static int Create_TT_SRPT
       } /*for*/
   } /*BuildAVInfo*/
 
-static int needmenus(const struct menugroup *mg)
+static bool needmenus(const struct menugroup *mg)
 /* do I actually have any menu definitions in mg. */
 {
-    if (!mg ) return 0;
-    if( !mg->numgroups ) return 0;
-    if( !mg->groups[0].pg->numpgcs ) return 0;
-    return 1;
+    if (!mg ) return false;
+    if( !mg->numgroups ) return false;
+    if( !mg->groups[0].pg->numpgcs ) return false; /* fixme: what about checking rest of groups? */
+    return true;
 }
 
 static void WriteIFO(FILE *h, const struct workset *ws)
@@ -544,7 +589,8 @@ static void WriteIFO(FILE *h, const struct workset *ws)
   {
     static unsigned char buf[2048];
     int nextsector;
-    const int forcemenus = needmenus(ws->menus);
+    const bool forcemenus = needmenus(ws->menus);
+    size_t ifo_pad = 0;
 
     // sect 0: VTS toplevel
     memset(buf, 0, 2048);
@@ -557,12 +603,12 @@ static void WriteIFO(FILE *h, const struct workset *ws)
     nextsector += Create_PTT_SRPT(0, ws->titles);
 
     write4(buf + 0xCC, nextsector); // VTS_PGCI
-    nextsector += CreatePGC(0, ws, 0);
+    nextsector += CreatePGC(0, ws, VTYPE_VTS);
 
     if (jumppad || forcemenus)
       {
         write4(buf + 0xD0,nextsector); // VTSM_PGCI
-        nextsector += CreatePGC(0, ws, 1);
+        nextsector += CreatePGC(0, ws, VTYPE_VTSM);
       } /*if*/
 
     write4(buf + 0xD4, nextsector); // VTS_TMAPT
@@ -571,54 +617,60 @@ static void WriteIFO(FILE *h, const struct workset *ws)
     if (jumppad || forcemenus)
       {
         write4(buf + 0xD8, nextsector); // VTSM_C_ADT
-        nextsector += CreateCellAddressTable(0, ws->menus->vg);
+        nextsector += CreateCellAddressTable(0, ws->menus->mg_vg);
 
         write4(buf + 0xDC, nextsector); // VTSM_VOBU_ADMAP
-        nextsector += numsectVOBUAD(ws->menus->vg);
+        nextsector += numsectVOBUAD(ws->menus->mg_vg);
       } /*if*/
 
     write4(buf + 0xE0, nextsector); // VTS_C_ADT
-    nextsector += CreateCellAddressTable(0, ws->titles->vg);
+    nextsector += CreateCellAddressTable(0, ws->titles->pg_vg);
 
     write4(buf + 0xE4, nextsector); // VTS_VOBU_ADMAP
-    nextsector += numsectVOBUAD(ws->titles->vg);
+    nextsector += numsectVOBUAD(ws->titles->pg_vg);
 
     write4(buf + 28, nextsector - 1); /* last sector of IFO */
     if (jumppad || forcemenus)
       {
         write4(buf + 0xC0, nextsector); /* start sector of menu VOB */
-        nextsector += getvoblen(ws->menus->vg);
+        nextsector += getvoblen(ws->menus->mg_vg);
       } /*if*/
     write4(buf + 0xC4, nextsector); /* start sector of title VOB */
     if (ws->titles->numpgcs)
-        nextsector += getvoblen(ws->titles->vg);
+        nextsector += getvoblen(ws->titles->pg_vg);
     nextsector += read4(buf + 28); /* offset by last sector of IFO */
+    if (nextsector < MIN_IFO_SECTORS - 1)
+      {
+        ifo_pad = (MIN_IFO_SECTORS - 1 - nextsector) * 2048;
+        nextsector = MIN_IFO_SECTORS - 1;
+      } /*if*/
     write4(buf + 12, nextsector); /* gives last sector of title set (last sector of BUP) */
 
     if (jumppad || forcemenus)
-        BuildAVInfo(buf + 256, ws->menus->vg);
-    BuildAVInfo(buf + 512, ws->titles->vg);
+        BuildAVInfo(buf + 256, ws->menus->mg_vg);
+    BuildAVInfo(buf + 512, ws->titles->pg_vg);
     nfwrite(buf, 2048, h);
 
     // sect 1: VTS_PTT_SRPT
     Create_PTT_SRPT(h, ws->titles);
-   
+
     // sect 2: VTS_PGCI
-    CreatePGC(h, ws, 0);
+    CreatePGC(h, ws, VTYPE_VTS);
 
     if( jumppad || forcemenus )
-        CreatePGC(h, ws, 1);
+        CreatePGC(h, ws, VTYPE_VTSM);
 
     // sect 3: ??? VTS_TMAPT
     CreateTMAPT(h, ws->titles);
 
     if (jumppad || forcemenus)
       {
-        CreateCellAddressTable(h, ws->menus->vg);
-        CreateVOBUAD(h, ws->menus->vg);
+        CreateCellAddressTable(h, ws->menus->mg_vg);
+        CreateVOBUAD(h, ws->menus->mg_vg);
       } /*if*/
-    CreateCellAddressTable(h, ws->titles->vg);
-    CreateVOBUAD(h, ws->titles->vg);
+    CreateCellAddressTable(h, ws->titles->pg_vg);
+    CreateVOBUAD(h, ws->titles->pg_vg);
+  /* nfpad(ifo_pad, h); */ /* unneeded, genisoimage will fix it for me */
   } /*WriteIFO*/
 
 void WriteIFOs(const char *fbase, const struct workset *ws)
@@ -626,20 +678,47 @@ void WriteIFOs(const char *fbase, const struct workset *ws)
   {
     FILE *h;
     static char buf[1000];
+    bool backup;
 
+    errno = 0;
     if (fbase)
       {
-        sprintf(buf, "%s_0.IFO", fbase);
-        h = fopen(buf, "wb");
-        WriteIFO(h, ws);
-          /* fixme: should do an fflush and check for write errors */
-        fclose(h);
-
-        sprintf(buf, "%s_0.BUP", fbase); /* same thing again, backup copy */
-        h = fopen(buf, "wb");
-        WriteIFO(h, ws);
-          /* fixme: should do an fflush and check for write errors */
-        fclose(h);
+        backup = false;
+        for (;;)
+          {
+            snprintf(buf, sizeof buf, "%s_0.%s", fbase, backup ? "BUP" : "IFO");
+            h = fopen(buf, "wb");
+            if (!h)
+              {
+                fprintf
+                  (
+                    stderr,
+                    "\nERR:  Error %d -- %s -- creating %s\n",
+                    errno,
+                    strerror(errno),
+                    buf
+                  );
+                exit(1);
+              } /*if*/
+            WriteIFO(h, ws);
+            fflush(h);
+            if (errno != 0)
+              {
+                fprintf
+                  (
+                    stderr,
+                    "\nERR:  Error %d -- %s -- flushing %s\n",
+                    errno,
+                    strerror(errno),
+                    buf
+                  );
+                exit(1);
+              } /*if*/
+            fclose(h);
+            if (backup)
+                break;
+            backup = true;
+          } /*for*/
       }
     else
       /* dummy write */
@@ -651,8 +730,9 @@ void TocGen(const struct workset *ws, const struct pgc *fpc, const char *fname)
   {
     static unsigned char buf[2048];
     int nextsector, offset, i, j, vtsstart;
-    const int forcemenus = needmenus(ws->menus);
+    const bool forcemenus = needmenus(ws->menus);
     FILE *h;
+    size_t ifo_pad = 0;
 
     h = fopen(fname, "wb");
 
@@ -663,7 +743,7 @@ void TocGen(const struct workset *ws, const struct pgc *fpc, const char *fname)
     buf[0x29] = 1; /* volume number */
     buf[0x2a] = 1; /* side ID */
     write2(buf + 0x3e, ws->titlesets->numvts); /* number of title sets */
-    strncpy((char *)(buf + 0x40), PACKAGE_STRING, 31); /* provider ID */
+    strncpy((char *)(buf + 0x40), provider_str, PROVIDER_SIZE - 1); /* provider ID */
     buf[0x86] = 4; /* start address of FP_PGC = 0x400 */
     nextsector = 1;
 
@@ -674,7 +754,7 @@ void TocGen(const struct workset *ws, const struct pgc *fpc, const char *fname)
     if (jumppad || forcemenus)
       {
         write4(buf + 0xc8, nextsector); /* sector pointer to VMGM_PGCI_UT (menu PGC table) */
-        nextsector += CreatePGC(0, ws, 2);
+        nextsector += CreatePGC(0, ws, VTYPE_VMGM);
       } /*if*/
 
     write4(buf + 0xd0, nextsector);
@@ -688,11 +768,11 @@ void TocGen(const struct workset *ws, const struct pgc *fpc, const char *fname)
         write4(buf + 0xd8, nextsector);
           /* sector pointer to VMGM_C_ADT (menu cell address table) */
           /* I make it follow VMG_VTS_ATRT */
-        nextsector += CreateCellAddressTable(0, ws->menus->vg); /* how much room it will need */
+        nextsector += CreateCellAddressTable(0, ws->menus->mg_vg); /* how much room it will need */
 
         write4(buf + 0xdc, nextsector);
           /* sector pointer to VMGM_VOBU_ADMAP (menu VOBU address map) */
-        nextsector += numsectVOBUAD(ws->menus->vg);
+        nextsector += numsectVOBUAD(ws->menus->mg_vg);
       } /*if*/
 
     write4(buf + 0x1c, nextsector - 1); /* last sector of IFO */
@@ -700,15 +780,20 @@ void TocGen(const struct workset *ws, const struct pgc *fpc, const char *fname)
     if (jumppad || forcemenus)
       {
         write4(buf + 0xc0, nextsector); /* start sector of menu VOB */
-        vtsstart += getvoblen(ws->menus->vg);
+        vtsstart += getvoblen(ws->menus->mg_vg);
+      } /*if*/
+    if (vtsstart < 2 * MIN_IFO_SECTORS)
+      {
+        ifo_pad = (2 * MIN_IFO_SECTORS - vtsstart) * 2048 / 2;
+        vtsstart = 2 * MIN_IFO_SECTORS;
       } /*if*/
     write4(buf + 0xc, vtsstart - 1); /* last sector of VMG set (last sector of BUP) */
 
     if (forcemenus)
-        BuildAVInfo(buf + 256, ws->menus->vg);
+        BuildAVInfo(buf + 256, ws->menus->mg_vg);
 
   /* create FPC at 0x400 as promised */
-    buf[0x407] = (getratedenom(ws->menus->vg) == 90090 ? 3 : 1) << 6;
+    buf[0x407] = (getratedenom(ws->menus->mg_vg) == 90090 ? 3 : 1) << 6;
       // only set frame rate XXX: should check titlesets if there is no VMGM menu
     buf[0x4e5] = 0xec; /* offset to command table, low byte */
     offset = 0x4f4; /* commands start here, after 8-byte header of command table */
@@ -780,7 +865,7 @@ void TocGen(const struct workset *ws, const struct pgc *fpc, const char *fname)
 
     // PGC
     if (jumppad || forcemenus)
-        CreatePGC(h, ws, 2);
+        CreatePGC(h, ws, VTYPE_VMGM);
 
   /* VMG_VTS_ATRT contains copies of menu and title attributes from all titlesets */
   /* output immediately following IFO header, as promised above */
@@ -804,16 +889,20 @@ void TocGen(const struct workset *ws, const struct pgc *fpc, const char *fname)
     j = 2048 - (j & 2047);
     if (j < 2048)
       { /* pad to next whole sector */
-        memset(buf, 0, j);
-        nfwrite(buf, j, h);
+        nfpad(j, h);
       } /*if*/
 
     if (jumppad || forcemenus)
       {
-        CreateCellAddressTable(h, ws->menus->vg); /* actually generate VMGM_C_ADT */
-        CreateVOBUAD(h, ws->menus->vg); /* generate VMGM_VOBU_ADMAP */
+        CreateCellAddressTable(h, ws->menus->mg_vg); /* actually generate VMGM_C_ADT */
+        CreateVOBUAD(h, ws->menus->mg_vg); /* generate VMGM_VOBU_ADMAP */
+      } /*if*/
+  /* nfpad(ifo_pad, h); */ /* unneeded, genisoimage will fix it for me */
+    fflush(h);
+    if (errno != 0)
+      {
+        fprintf(stderr, "\nERR:  Error %d -- %s -- flushing VMGM\n", errno, strerror(errno));
+        exit(1);
       } /*if*/
-  /* fixme: should do an fflush and check for write errors */
     fclose(h);
   } /*TocGen*/
-
