@@ -16,14 +16,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
- * USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301 USA.
  */
 
-#include "config.h"
-
 #include "compat.h"
-
+#include <errno.h>
 #include <assert.h>
 
 #include "dvdauthor.h"
@@ -66,7 +64,7 @@ static int genjumppad(unsigned char *buf, vtypes ismenu, int entry, const struct
   {
     unsigned char *cbuf = buf;
     int i, j, k;
-    if (jumppad && ismenu == VTYPE_VTSM && entry == 7)
+    if (jumppad && ismenu == VTYPE_VTSM && entry == 7 /* root menu? */)
       {
         // *** VTSM jumppad
         write8(cbuf,0x61,0x00,0x00,0x0E,0x00,0x0F,0x00,0x00); cbuf+=8; // g[14]=g[15];
@@ -97,7 +95,7 @@ static int genjumppad(unsigned char *buf, vtypes ismenu, int entry, const struct
               } /*for*/
           } /*for*/
       }
-    else if (jumppad && ismenu == VTYPE_VMGM && entry == 2)
+    else if (jumppad && ismenu == VTYPE_VMGM && entry == 2 /* title menu */)
       {
         // *** VMGM jumppad
         // remap all VMGM TITLE X -> TITLESET X TITLE Y
@@ -160,7 +158,7 @@ static int jumppgc(unsigned char *buf,int pgc,vtypes ismenu,int entry,const stru
 static int genpgc(unsigned char *buf,const struct workset *ws,const struct pgcgroup *group,int pgc,vtypes ismenu,int entry)
 /* generates a PGC entry for an IFO file in buf. */
   {
-    const struct vobgroup *va = (ismenu != VTYPE_VTS ? ws->menus->vg : ws->titles->vg);
+    const struct vobgroup *va = (ismenu != VTYPE_VTS ? ws->menus->mg_vg : ws->titles->pg_vg);
     const struct pgc * const thispgc = group->pgcs[pgc];
     int i, j, d;
 
@@ -180,15 +178,16 @@ static int genpgc(unsigned char *buf,const struct workset *ws,const struct pgcgr
       } /*for*/
     for (i = 0; i < va->numsubpicturetracks; i++)
       {
-        int m, e;
-        e = 0;
+        int m;
+        bool e;
+        e = false;
         for (m = 0; m < 4; m++)
             if (thispgc->subpmap[i][m] & 128)
               {
                 buf[28 + i * 4 + m] = thispgc->subpmap[i][m] & 127;
                   /* PGC_SPST_CTL, subpicture stream control: actual stream IDs corresponding
                     to each stream described in IFO */
-                e = 1;
+                e = true;
               } /*if; for*/
         if (e)
             buf[28 + i * 4] |= 0x80; /* set stream-available flag */
@@ -305,7 +304,7 @@ static int genpgc(unsigned char *buf,const struct workset *ws,const struct pgcgr
                     cd = cdd;
                   } /*if*/
               } /*for; for*/
-        
+
         write2(buf + 228, d); /* offset to commands */
         if (cd - (buf + d) - 8 > 128 * 8) // can only have 128 commands
           {
@@ -323,7 +322,8 @@ static int genpgc(unsigned char *buf,const struct workset *ws,const struct pgcgr
     if (thispgc->numsources)
       {
       /* fill in program map, cell playback info table and cell position info table */
-        int cellline, notseamless;
+        int cellline;
+        bool notseamless;
         write2(buf + 230, d); /* offset to program map */
         j = 1;
         for (i = 0; i < thispgc->numsources; i++)
@@ -344,7 +344,7 @@ static int genpgc(unsigned char *buf,const struct workset *ws,const struct pgcgr
         write2(buf + 232, d); /* offset to cell playback information table */
         j = -1;
         cellline = 1;
-        notseamless = 1; /* first cell never seamless, otherwise a/v sync problems will occur */
+        notseamless = true; /* first cell never seamless, otherwise a/v sync problems will occur */
         for (i = 0; i < thispgc->numsources; i++)
           { /* generate the cell playback information table */
             int k, l, m, firsttime;
@@ -363,13 +363,13 @@ static int genpgc(unsigned char *buf,const struct workset *ws,const struct pgcgr
                             &&
                                 calcaudiogap(va, j, id, getaudch(va, m))
                           )
-                            notseamless = 1;
+                            notseamless = true;
                     buf[d] =
                             (notseamless ? 0 : 8 /* seamless multiplex */)
                         |
                             (j + 1 != id ? 2 /* SCR discontinuity */ : 0);
                               // if this is the first cell of the source, then set 'STC_discontinuity', otherwise set 'seamless presentation'
-                    notseamless = 0; /* initial assumption for next cell */
+                    notseamless = false; /* initial assumption for next cell */
                     j = id;
 
                     vi = findcellvobu(thissource->vob, l);
@@ -383,12 +383,12 @@ static int genpgc(unsigned char *buf,const struct workset *ws,const struct pgcgr
                         if (thissource->cells[k].pauselen > 0)
                           {
                             buf[2 + d] = thissource->cells[k].pauselen; /* cell still time */
-                            notseamless = 1; // cells with stilltime are not seamless
+                            notseamless = true; // cells with stilltime are not seamless
                           } /*if*/
                         if (thissource->cells[k].commands)
                           {
                             buf[3 + d] = cellline++; /* cell command nr */
-                            notseamless = 1; // cells with commands are not seamless
+                            notseamless = true; // cells with commands are not seamless
                           } /*if*/
                       } /*if*/
                     write4(buf + 4 + d, buildtimeeven(va, thissource->vob->vobu[vi]. sectpts[1] - firsttime));
@@ -420,40 +420,45 @@ static int genpgc(unsigned char *buf,const struct workset *ws,const struct pgcgr
   } /*genpgc*/
 
 static int createpgcgroup(const struct workset *ws,vtypes ismenu,const struct pgcgroup *va,unsigned char *buf /* where to put generated table */)
-  /* generates a VMGM_LU or VTSM_LU table and all associated PGCs. Returns -1 if there wasn't enough space. */
+  /* generates a VMGM_LU, VTSM_LU or VTS_PGCI table and all associated PGCs. Returns -1 if there wasn't enough space. */
   {
-    int len,i,pgcidx;
-
+    int len, i, pgcidx, nrtitles;
     len = va->numpgcs + va->numentries;
+      /* duplicate each PGC for each additional menu entry type it may have ... */
     for (i = 0; i < va->numpgcs; i++)
         if (va->pgcs[i]->entries)
-            len--;
-    write2(buf, len); /* nr program chains */
-    len = len * 8 + 8; /* total length of VMGM_LU/VTSM_LU */
+            len--; /* ...  beyond the first one */
+    write2(buf, len); /* nr language units (VMGM_LU, VTSM_LU)/program chains (VTS_PGCI) */
+    len = len * 8 + 8; /* total length of VMGM_LU/VTSM_LU/VTS_PGCI */
     pgcidx = 8;
-
+    nrtitles = 0;
     for (i = 0; i < va->numpgcs; i++)
       { /* generate the PGCs and put in their offsets */
         int j = 0;
         if (buf + len + BUFFERPAD - bigwritebuf > bigwritebuflen)
             return -1; /* caller needs to give me more space */
         if (ismenu == VTYPE_VTS)
-            buf[pgcidx] = 0x81 + i; /* menu type for VTSM_LU */
+          {
+            if (va->pgcs[i]->entries == 0)
+              {
+                ++nrtitles;
+                buf[pgcidx] = 0x80 | nrtitles; /* title type & nr for VTS_PGC */
+              } /*if*/
+          }
         else
           {
             buf[pgcidx] = 0;
             for (j = 2; j < 8; j++)
                 if (va->pgcs[i]->entries & (1 << j))
                   {
-                    buf[pgcidx] = 0x80 | j; /* menu type for VMGM_LU */
+                    buf[pgcidx] = 0x80 | j; /* menu type for VMGM_LU/VTSM_LU */
                     break;
                   } /*if; for*/
           } /*if*/
-        write4(buf + pgcidx + 4, len); /* offset to VMGM_PGC/VTSM_PGC */
+        write4(buf + pgcidx + 4, len); /* offset to VMGM_PGC/VTSM_PGC/VTS_PGC */
         len += genpgc(buf + len, ws, va, i, ismenu, j); /* put it there */
         pgcidx += 8;
       } /*for*/
-
     for (i = 2; i < 8; i++)
       {
         if (va->allentries & (1 << i))
@@ -479,7 +484,6 @@ static int createpgcgroup(const struct workset *ws,vtypes ismenu,const struct pg
             pgcidx += 8;
           } /*if*/
       } /*for*/
-
     write4(buf + 4, len - 1);
       /* end address (last byte of last PGC in this LU) relative to VMGM_LU/VTSM_LU */
     return len;
@@ -488,9 +492,10 @@ static int createpgcgroup(const struct workset *ws,vtypes ismenu,const struct pg
 int CreatePGC(FILE *h, const struct workset *ws, vtypes ismenu)
   {
     unsigned char *buf;
-    int len,ph,i,in_it;
+    int len,ph,i;
+    bool in_it;
 
-    in_it = !bigwritebuflen; /* don't do first allocation if already got a buffer from last time */
+    in_it = bigwritebuflen == 0; /* don't do first allocation if already got a buffer from last time */
  retry: /* come back here if buffer wasn't big enough to generate a PGC group */
     if (in_it)
       {
@@ -502,10 +507,9 @@ int CreatePGC(FILE *h, const struct workset *ws, vtypes ismenu)
             free(bigwritebuf);
         bigwritebuf = malloc(bigwritebuflen);
       } /*if*/
-    in_it = 1;
+    in_it = true;
     buf = bigwritebuf;
     memset(buf, 0, bigwritebuflen);
-    ph = 0;
     if (ismenu != VTYPE_VTS) /* create VMGM_PGCI_UT/VTSM_PGCI_UT structure */
       {
         buf[1] = ws->menus->numgroups; // # of language units
@@ -516,9 +520,9 @@ int CreatePGC(FILE *h, const struct workset *ws, vtypes ismenu)
             unsigned char * const plu = buf + 8 + 8 * i;
             const struct langgroup * const lg = &ws->menus->groups[i];
             memcpy(plu, lg->lang, 2); /* ISO639 language code */
-            if (ismenu == 1)
+            if (ismenu == VTYPE_VTSM)
                 plu[3] = lg->pg->allentries;
-            else
+            else /* ismenu = VTYPE_VMGM */
                 plu[3] = 0x80; // menu system contains entry for title
             write4(plu + 4, ph);
             len = createpgcgroup(ws, ismenu, lg->pg, buf + ph);
@@ -532,7 +536,7 @@ int CreatePGC(FILE *h, const struct workset *ws, vtypes ismenu)
     else
       {
       /* generate VTS_PGCI structure */
-        len = createpgcgroup(ws, 0, ws->titles, buf);
+        len = createpgcgroup(ws, VTYPE_VTS, ws->titles, buf);
         if (len < 0)
             goto retry;
         ph = len;
@@ -541,6 +545,19 @@ int CreatePGC(FILE *h, const struct workset *ws, vtypes ismenu)
     assert(ph <= bigwritebuflen);
     ph = (ph + 2047) & (-2048);
     if (h)
-        fwrite(buf, 1, ph, h);
+      {
+        (void)fwrite(buf, 1, ph, h);
+        if (errno != 0)
+          {
+            fprintf
+              (
+                stderr,
+                "\nERR:  Error %d -- %s -- writing output PGC\n",
+                errno,
+                strerror(errno)
+              );
+            exit(1);
+          } /*if*/
+      } /*if*/
     return ph / 2048;
   } /*CreatePGC*/
