@@ -1,15 +1,13 @@
 #include <stdlib.h>
 #include <string.h>
 
-#define YY_NO_INPUT
-
 #ifdef YY_END_OF_BUFFER_CHAR
 #define dvdvm_buffer_state YY_BUFFER_STATE
 #else
 typedef void *dvdvm_buffer_state;
 #endif
 
-extern int allowallreg;
+extern bool allowallreg;
 
 extern void dvdvmerror(const char *s);
 extern int dvdvmlex(void);
