@@ -17,12 +17,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
- * USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301 USA.
  */
 
 // prog for encoding dvd subtitles and multiplexing them into an mpeg
 // available under GPL v2
+/* Thanks to TED for sponsoring implementation of --nomux and --nodvdauthor-data options */
 
 #include "config.h"
 
@@ -35,22 +36,28 @@
 #include <netinet/in.h>
 
 #include "common.h"
+#include "conffile.h"
 #include "rgb.h"
+#include "subglobals.h"
+#include "subrender.h"
 #include "subgen.h"
-#include "textsub.h"
 
 // (90000*300)/(1260000/2048)
 // (9*300)/(126/2048)
 // (9*300*2048)/126
 #define DVDRATE 43886
 
+/* all the subpicture formats follow the DVD-Video convention of using
+MPEG Private Stream 1, where the first byte of the packet content indicates
+the subpicture stream ID, based off the following values */
 #define CVD_SUB_CHANNEL     0x0
-#define SVCD_SUB_CHANNEL    0x70
+#define SVCD_SUB_CHANNEL    0x70 /* fixed value, actual stream ID is next byte */
 #define DVD_SUB_CHANNEL     0x20
 
-#define DVD_SUB     0
-#define CVD_SUB     1
-#define SVCD_SUB    2
+/* formats supported: */
+#define DVD_SUB     0 /* DVD-Video */
+#define CVD_SUB     1 /* China Video Disc */
+#define SVCD_SUB    2 /* Super Video CD */
 
 #define psbufs 10
 
@@ -59,33 +66,39 @@
 
 static unsigned char *cbuf;
 
-static unsigned int spuindex, progr;
-static int tofs;
-static int svcd;
+static unsigned int spuindex;
+static bool
+    show_progress = false,
+    dodvdauthor_data = true,
+    domux = true;
+static int tofs; /* timestamp of first video packet, for synchronizing subpicture timestamps */
+static int svcd_adjust;
 
-static uint64_t lps;
+static uint64_t lps; /* output bytes written */
+
+int default_video_format = VF_NONE;
+bool widescreen = false;
 
 stinfo **spus=0;
 int numspus=0;
-int have_textsub=0;
-int have_transparent=1;
-int transparent_color=0x808080;
+bool have_textsub = false;
 
-int skip;
+int nr_subtitles_skipped;
 
 static char header[32];
 
 unsigned char *sub;
 int debug;
-int max_sub_size;
+static int max_sub_size;
+
+static bool substream_present[256];
 
 
-
-// these 4 lines of variables are used by mux() and main() to communicate
+// these 4 lines of variables are used by muxnext() and main() to communicate
 static int subno,secsize,mode,fdo,header_size,muxrate;
 static unsigned char substr, *sector;
 static stinfo *newsti;
-static uint64_t gts, nextgts;
+static uint64_t lastgts, nextgts;
 
 
 /*
@@ -141,7 +154,7 @@ static void mkpackh
     unsigned int muxrate /* data rate in units of 50 bytes/second */,
     unsigned char stuffing /* nr stuffing bytes to follow, [0 .. 7] */
   )
-/* constructs the contents for a PACK header. */
+/* constructs the contents for a PACK header in header. */
 {
     unsigned long const th = time / 300, tl = time % 300;
     header[0] = 0x44 | ((th >> 27) & 0x38) | ((th >> 28) & 3);
@@ -157,7 +170,8 @@ static void mkpackh
 }
 
 static void mkpesh0(unsigned long int pts)
-/* constructs an MPEG-2 PES header extension with PTS data but no PES extension. */
+  /* constructs an MPEG-2 PES header extension with PTS data but no PES extension
+    in header. */
 {
     header[0] = 0x81; /* original flag set */
     header[1] = 0x80;   //pts flag
@@ -175,7 +189,7 @@ static void mkpesh1(unsigned long int pts)
     header[0] = 0x81; /* original flag set */
     header[1] = 0x81;   //pts flag + pes extension flag
     header[2] = 8; /* PES header data length */
-    header[3] = 0x21 | ((pts >> 29)&6); /* PTS[31 .. 30] */
+    header[3] = 0x21 | ((pts >> 29) & 6); /* PTS[31 .. 30] */
     header[4] = pts >> 22; /* PTS[29 .. 22] */
     header[5] = 1 | (pts >> 14); /* PTS[21 .. 15] */
     header[6] = (pts >> 7); /* PTS[14 .. 7] */
@@ -186,7 +200,7 @@ static void mkpesh1(unsigned long int pts)
 }
 
 static void mkpesh2 ()
-/* constructs an empty MPEG-2 PES header extension. */
+/* constructs an empty MPEG-2 PES header extension in header. */
 {
     header[0] = 0x81; /* original flag set */
     header[1] = 0; /* nothing else */
@@ -194,10 +208,11 @@ static void mkpesh2 ()
 }
 
 static unsigned int getmuxr(const unsigned char *buf)
-/* obtains the muxrate value from the contents of a PACK header. */
-{
-    return (buf[8] >> 2)|(buf[7]*64)|(buf[6]*16384);
-}
+  /* obtains the muxrate value (data rate in units of 50 bytes/second) from the
+    contents of a PACK header. */
+  {
+    return (buf[8] >> 2) | (buf[7] * 64) | (buf[6] * 16384);
+  } /*getmuxr*/
 
 static uint64_t getgts(const unsigned char *buf)
   /* returns the timestamp from the contents of a PACK header. This will be
@@ -239,6 +254,10 @@ static uint64_t getgts(const unsigned char *buf)
   } /*getgts*/
 
 static void fixgts(uint64_t *gts, uint64_t *nextgts)
+  /* ensures that *gts is increasing in steps of at least DVDRATE
+    on successive calls. *nextgts is the expected minimum value
+    computed on the previous call, will be updated to the expected
+    minimum for the next call. */
   {
     if (gts[0] < nextgts[0])
         gts[0] = nextgts[0];
@@ -274,12 +293,12 @@ static unsigned int getpts(const unsigned char *buf)
             (buf[3] & 14) * 16384 * 256 * 128;
   } /*getpts*/
 
-int findmasterpal(stinfo *s, const palt *p)
+int findmasterpal(stinfo *s, const colorspec *p)
   /* returns the index in s->masterpal corresponding to colour p, allocating a
     new palette entry if not there already. */
   {
     int i;
-    if (!p->t)
+    if (!p->a)
         return 0;
     for (i = 0; i < s->numpal; i++)
         if
@@ -319,31 +338,43 @@ static void freestinfo(stinfo *s)
     free(s);
 }
 
-int calcY(const palt *p)
+int calcY(const colorspec *p)
 {
     return RGB2Y(p->r,p->g,p->b);
 }
 
-int calcCr(const palt *p)
+int calcCr(const colorspec *p)
 {
     return RGB2Cr(p->r,p->g,p->b);
 }
 
-int calcCb(const palt *p)
+int calcCb(const colorspec *p)
 {
     return RGB2Cb(p->r,p->g,p->b);
 }
 
 static unsigned char *wdest;
+static const unsigned char * wdest_end;
+
+static void wdest_need(size_t nrbytes)
+  {
+    if (wdest + nrbytes > wdest_end)
+      {
+        fprintf(stderr, "ERR:  too much data for dvdauthor-data sector buffer\n");
+        exit(1);
+      } /*if*/
+  } /*wdest_need*/
 
 static void wdbyte(unsigned char c)
 {
+    wdest_need(1);
     wdest[0]=c;
     wdest++;
 }
 
 static void wdshort(unsigned short s)
 {
+    wdest_need(2);
     wdest[0]=s>>8;
     wdest[1]=s;
     wdest+=2;
@@ -351,6 +382,7 @@ static void wdshort(unsigned short s)
 
 static void wdlong(unsigned long l)
 {
+    wdest_need(4);
     wdest[0]=l>>24;
     wdest[1]=l>>16;
     wdest[2]=l>>8;
@@ -410,7 +442,7 @@ static void swrite(int h, const void *b, int l)
 static stinfo *getnextsub(void)
   /* processes and returns the next subtitle definition, if there is one. */
   {
-    while(1)
+    while (true)
       {
         stinfo *s;
         if (spuindex >= numspus) /* no more to return */
@@ -432,7 +464,7 @@ static stinfo *getnextsub(void)
         if (process_subtitle(s))
             return s;
         freestinfo(s);
-        skip++;
+        nr_subtitles_skipped++;
       } /*while*/
   } /*getnextsub*/
 
@@ -447,22 +479,22 @@ static void usage()
     exit(-1);
 }
 
-static void mux(int eoinput)
+static void muxnext(bool eoinput)
   {
-    if (gts == 0 || tofs == -1 || (lps % secsize && !eoinput))
+    if (domux && (lastgts == 0 || tofs == -1 || (lps % secsize && !eoinput)))
         return;
     while (newsti)
       {
         stinfo *cursti;
-        int bytes_send, sub_size;
+        int bytes_sent, sub_size;
         unsigned char seq;
         unsigned int q;
-        int64_t dgts;
+        int64_t duegts;
       /* wait for correct time to insert sub, leave time for vpts to occur */
-        dgts = (newsti->spts - .15 * 90000) * 300;
-        if (dgts < 0)
-            dgts = 0;
-        if (dgts > gts && !eoinput)
+        duegts = (newsti->spts - .15 * 90000) * 300;
+        if (duegts < 0)
+            duegts = 0;
+        if (domux && duegts > lastgts && !eoinput)
             break; /* not yet time */
         cursti = newsti;
         if (debug > 1)
@@ -511,7 +543,7 @@ static void mux(int eoinput)
                         cursti->spts / 90000, cursti->sd / 90000);
               } /*if*/
           } /*if*/
-        if ((cursti->sd == -1) && newsti && ((!svcd) || until_next_sub))
+        if (cursti->sd == -1 && newsti && (!svcd_adjust || until_next_sub))
           {
             if (newsti->spts > cursti->spts + tbs)
                 cursti->sd = newsti->spts - cursti->spts - tbs;
@@ -520,15 +552,14 @@ static void mux(int eoinput)
                 if (debug > -1)
                   {
                     fprintf(stderr,\
-                            "ERR: Sub with too short or negative duration on line %d, skipping\n",\
+                            "WARN:  Sub with too short or negative duration on line %d, skipping\n",\
                             spuindex - 1);
                   } /*if*/
-                exit(1);
-                skip++;
+                nr_subtitles_skipped++;
                 continue;
               } /*if*/
           } /*if*/
-        switch(mode)
+        switch (mode)
           {
         case DVD_SUB:
           /* rle here */
@@ -550,169 +581,183 @@ static void mux(int eoinput)
               {
                 fprintf(stderr, "WARN: Image too large (encoded size>64k), skipping line %d\n", spuindex - 1);
               } /*if*/
-            skip++;
+            nr_subtitles_skipped++;
             continue;
           } /*if*/
         if (sub_size > max_sub_size)
           {
             max_sub_size = sub_size;
-            if (have_textsub == 0)
+            if (!have_textsub)
                 fprintf(stderr, "INFO: Max_sub_size=%d\n", max_sub_size);
           } /*if*/
         seq = 0;
         subno++;
-        gts = dgts;
+        lastgts = duegts;
         if (mode == DVD_SUB)
           {
-          /* write out custom dvdauthor information */
-            int pdl = secsize - 6 - 10 - 4, i;
-            unsigned int c;
-          /* write packet start code */
-            c = htonl(0x100 + MPID_PACK);
-            swrite(fdo, &c, 4);
-            mkpackh(gts, muxrate, 0);
-            fixgts(&gts, &nextgts);
-            swrite(fdo, header, 10);
-            // start padding streamcode
-            header[0] = 0;
-            header[1] = 0;
-            header[2] = 1;
-            header[3] = MPID_PAD; /* for my private button/palette data */
-            header[4] = pdl >> 8;
-            header[5] = pdl;
-            swrite(fdo, header, 6);
-
-            memset(sector, 0xff, pdl);
-
-            wdest = sector;
-            wdstr("dvdauthor-data");
-            wdbyte(2); // version
-            wdbyte(1); // subtitle info
-            wdbyte(substr); // sub number
-            wdlong(cursti->spts); // start pts
-            wdlong(cursti->sd == -1 ? -1 : cursti->sd + cursti->spts); // end pts
-
-            wdbyte(1); // colormap
-            wdbyte(cursti->numpal); // number of colors
-            for (i = 0; i < cursti->numpal; i++)
+            if (dodvdauthor_data)
               {
-                wdbyte(calcY(cursti->masterpal + i));
-                wdbyte(calcCr(cursti->masterpal + i));
-                wdbyte(calcCb(cursti->masterpal + i));
-              } /*for*/
+              /* write out custom dvdauthor information */
+                int pdl = secsize - 6 - 10 - 4, i;
+                unsigned int c;
+              /* write packet start code */
+                c = htonl(0x100 + MPID_PACK);
+                swrite(fdo, &c, 4);
+                mkpackh(lastgts, muxrate, 0);
+                fixgts(&lastgts, &nextgts);
+                swrite(fdo, header, 10);
+                // start padding streamcode
+                header[0] = 0;
+                header[1] = 0;
+                header[2] = 1;
+                header[3] = MPID_PAD; /* for my private button/palette data */
+                header[4] = pdl >> 8;
+                header[5] = pdl;
+                swrite(fdo, header, 6);
 
-            if (cursti->numgroups)
-              {
-                wdbyte(2); // st_coli
-                wdbyte(cursti->numgroups);
-                for (i = 0; i < cursti->numgroups; i++)
+                memset(sector, 0xff, pdl);
+
+                wdest = sector;
+                wdest_end = sector + secsize;
+                wdstr("dvdauthor-data");
+                wdbyte(2); // version
+                wdbyte(1); // subtitle info
+                wdbyte(substr); // sub number
+                wdlong(cursti->spts); // start pts
+                wdlong(cursti->sd == -1 ? -1 : cursti->sd + cursti->spts); // end pts
+
+                wdbyte(1); // colormap
+                wdbyte(cursti->numpal); // number of colors
+                for (i = 0; i < cursti->numpal; i++)
                   {
-                    unsigned short sh[4];
-                    int j;
-                    for (j = 3; j >= 0; j--)
+                    wdbyte(calcY(cursti->masterpal + i));
+                    wdbyte(calcCr(cursti->masterpal + i));
+                    wdbyte(calcCb(cursti->masterpal + i));
+                  /* I don't need to pass alpha, because that has already been encoded
+                    into the subpicture stream with SPU_SET_CONTR commands */
+                  } /*for*/
+
+                if (cursti->numgroups)
+                  {
+                    wdbyte(2); // st_coli
+                    wdbyte(cursti->numgroups);
+                    for (i = 0; i < cursti->numgroups; i++)
                       {
-                        int k = cursti->groupmap[i][j];
-                        if (k == -1)
+                        unsigned short sh[4];
+                        int j;
+                        for (j = 3; j >= 0; j--)
                           {
-                            for (k = 0; k < 4; k++)
-                                sh[k] <<= 4;
-                          }
-                        else
-                          {
-                            sh[0] =
-                                    sh[0] << 4
-                                |
-                                    findmasterpal(cursti, cursti->hlt.pal + (k >> 8 & 255));
-                            sh[1] =
-                                    sh[1] << 4
-                                |
-                                    cursti->hlt.pal[k >> 8 & 255].t >> 4;
-                            sh[2] =
-                                    sh[2] << 4
-                                |
-                                    findmasterpal(cursti, cursti->sel.pal + (k & 255));
-                            sh[3] =
-                                    sh[3] << 4
-                                |
-                                    cursti->sel.pal[k & 255].t >> 4;
-                          } /*if*/
+                            int k = cursti->groupmap[i][j];
+                            if (k == -1)
+                              {
+                                for (k = 0; k < 4; k++)
+                                    sh[k] <<= 4;
+                              }
+                            else
+                              {
+                                sh[0] =
+                                        sh[0] << 4
+                                    |
+                                        findmasterpal(cursti, cursti->hlt.pal + (k >> 8 & 255));
+                                sh[1] =
+                                        sh[1] << 4
+                                    |
+                                        cursti->hlt.pal[k >> 8 & 255].a >> 4;
+                                sh[2] =
+                                        sh[2] << 4
+                                    |
+                                        findmasterpal(cursti, cursti->sel.pal + (k & 255));
+                                sh[3] =
+                                        sh[3] << 4
+                                    |
+                                        cursti->sel.pal[k & 255].a >> 4;
+                              } /*if*/
+                          } /*for*/
+                        for (j = 0; j < 4; j++)
+                            wdshort(sh[j]);
                       } /*for*/
-                    for (j = 0; j < 4; j++)
-                        wdshort(sh[j]);
-                  } /*for*/
-              } /*if*/
-            if (cursti->numbuttons)
-              {
-                wdbyte(3);
-                wdbyte(cursti->numbuttons);
-                for (i = 0; i < cursti->numbuttons; i++)
+                  } /*if*/
+                if (cursti->numbuttons)
                   {
-                    const button * const b = &cursti->buttons[i];
-                    char nm1[10], nm2[10];
-                    wdstr(b->name);
-                    wdshort(0);
-                    wdbyte(b->autoaction);
-                    wdbyte(b->grp);
-                    wdshort(b->r.x0);
-                    wdshort(b->r.y0);
-                    wdshort(b->r.x1);
-                    wdshort(b->r.y1);
-                    if ((b->r.y0 & 1) || (b->r.y1 & 1))
-                        fprintf
-                          (
-                            stderr,
-                            "WARN: Button y coordinates are odd for button %s: %dx%d-%dx%d;"
-                                " they may not display properly.\n",
-                            b->name,
-                            b->r.x0,
-                            b->r.y0,
-                            b->r.x1,
-                            b->r.y1
-                          );
-                    sprintf(nm1, "%d", i ? i : (cursti->numbuttons));
-                    sprintf(nm2, "%d", (i + 1 != cursti->numbuttons) ? (i + 2) : 1);
-                    // fprintf(stderr,"BUTTON NAVIGATION FOR %s: up=%s down=%s left=%s right=%s (%s %s)\n",b->name,b->up,b->down,b->left,b->right,nm1,nm2);
-                  /* fixme: no constraints on length of button names */
-                    wdstr(b->up ? b->up : nm1);
-                    wdstr(b->down ? b->down : nm2);
-                    wdstr(b->left ? b->left : nm1);
-                    wdstr(b->right ? b->right : nm2);
-                  } /*for*/
+                    wdbyte(3);
+                    wdbyte(cursti->numbuttons);
+                    for (i = 0; i < cursti->numbuttons; i++)
+                      {
+                        const button * const b = &cursti->buttons[i];
+                        char nm1[10], nm2[10];
+                        wdstr(b->name);
+                        wdshort(0);
+                        wdbyte(b->autoaction ? 1 : 0);
+                        wdbyte(b->grp);
+                        wdshort(b->r.x0);
+                        wdshort(b->r.y0);
+                        wdshort(b->r.x1);
+                        wdshort(b->r.y1);
+                        if ((b->r.y0 & 1) || (b->r.y1 & 1))
+                            fprintf
+                              (
+                                stderr,
+                                "WARN: Button y coordinates are odd for button %s: %dx%d-%dx%d;"
+                                    " they may not display properly.\n",
+                                b->name,
+                                b->r.x0,
+                                b->r.y0,
+                                b->r.x1,
+                                b->r.y1
+                              );
+                        sprintf(nm1, "%d", i ? i : (cursti->numbuttons));
+                        sprintf(nm2, "%d", (i + 1 != cursti->numbuttons) ? (i + 2) : 1);
+                        // fprintf(stderr,"BUTTON NAVIGATION FOR %s: up=%s down=%s left=%s right=%s (%s %s)\n",b->name,b->up,b->down,b->left,b->right,nm1,nm2);
+                      /* fixme: no constraints on length of button names */
+                        wdstr(b->up ? b->up : nm1);
+                        wdstr(b->down ? b->down : nm2);
+                        wdstr(b->left ? b->left : nm1);
+                        wdstr(b->right ? b->right : nm2);
+                      } /*for*/
+                  } /*if*/
+              /* fprintf(stderr,"INFO: Private sector size %d\n",wdest-sector); */
+                swrite(fdo, sector, pdl);
+              }
+            else
+              {
+                if (cursti->numbuttons)
+                  {
+                    fprintf(stderr, "WARN: Button info not being passed to dvdauthor.\n");
+                  } /*if*/
               } /*if*/
-          /* fprintf(stderr,"INFO: Private sector size %d\n",wdest-sector); */
-            swrite(fdo, sector, pdl);
           } /*if mode == DVD_SUB*/
         // header_size is 12 before while starts
+      /* header_size includes first byte of PES data, i.e. substream ID */
       /* search packet start code */
-        bytes_send = 0;
-        while (bytes_send != sub_size)
+        bytes_sent = 0;
+        while (bytes_sent != sub_size)
           {
-            int i, stuffing;
+            int bytes_this_packet, stuffing;
             uint32_t c;
             uint16_t b;
           /* if not first time here */
-            if (bytes_send)
-                header_size = 4;
-            else if (header_size != 12)
-                header_size = 9; // not first time
+            if (bytes_sent)
+                header_size = 4; /* empty MPEG-2 PES header extension on continuation packet */
+            else if (header_size != 12) // not first time
+                header_size = 9; /* drop PES extension from subsequent packets */
           /* calculate how many bytes to send */
-            i = secsize - 20 - header_size - svcd;
-            stuffing = i - (sub_size - bytes_send);
+            bytes_this_packet = secsize - 20 - header_size - svcd_adjust;
+            stuffing = bytes_this_packet - (sub_size - bytes_sent);
             if ( stuffing < 0)
                 stuffing = 0;
             else
               {
-                i -= stuffing;
+                bytes_this_packet -= stuffing;
                 if (stuffing > 7)
                     stuffing = 0;
               } /*if*/
           /* write header */
             c = htonl(0x100 + MPID_PACK);
             swrite(fdo, &c, 4);
-            mkpackh(gts, muxrate, 0);
-            fixgts(&gts, &nextgts);
+            mkpackh(lastgts, muxrate, 0);
+            fixgts(&lastgts, &nextgts);
 /*
-  fprintf(stderr, "system time: %d 0x%lx %d\n", gts, ftell(fds), frame);
+  fprintf(stderr, "system time: %d 0x%lx %d\n", lastgts, ftell(fds), frame);
   fprintf(stderr, "spts=%d\n", spts);
 */
             swrite(fdo, header, 10);
@@ -720,37 +765,40 @@ static void mux(int eoinput)
             c = htonl(0x100 + MPID_PRIVATE1);
             swrite(fdo, &c, 4);
           /* write packet length */
-            b = ntohs(i + header_size + svcd + stuffing);
+            b = ntohs(bytes_this_packet + header_size + svcd_adjust + stuffing);
             swrite(fdo, &b, 2);
-          /* i has NOT changed here! and is still bytes to send */
             if (header_size == 9)
                 mkpesh0(cursti->spts);
             else if (header_size == 12)
                 mkpesh1(cursti->spts);
-            else
+            else /* header_size = 4 */
                 mkpesh2();
-            header[2] += stuffing;
+            header[2] += stuffing; /* include in PES header data size */
             memset(header + header_size - 1, 0xff, stuffing);
-            header[header_size + stuffing - 1] = svcd ? SVCD_SUB_CHANNEL : substr; /* subpicture stream number */
+            header[header_size + stuffing - 1] = /* substream ID */
+                svcd_adjust ?
+                    SVCD_SUB_CHANNEL /* real subpicture stream number inserted below */
+                :
+                    substr; /* subpicture stream number */
             swrite(fdo, header, header_size + stuffing);
-            if (svcd)
+            if (svcd_adjust)
               {
-              /* 4 byte svcd header */
+              /* additional 4 byte svcd header */
                 const uint16_t cc = htons(subno);
-                swrite(fdo, &substr, 1); // current subtitle stream
-                if (bytes_send + i == sub_size)
-                    seq |= 128;
+                swrite(fdo, &substr, 1); /* real subpicture stream number */
+                if (bytes_sent + bytes_this_packet == sub_size)
+                    seq |= 128; /* end of current sub */
                 swrite(fdo, &seq, 1); // packet number in current sub
                 // 0 - up, last packet has bit 7 set
                 swrite(fdo, &cc, 2);
               } /*if*/
-            seq++;
-          /* write i data bytes, increment bytes_send by bytes written */
-            swrite(fdo, sub + bytes_send, i);
-            bytes_send += i;
+            seq++; /* won't count past 127? */
+          /* write bytes_this_packet data bytes, increment bytes_sent by bytes written */
+            swrite(fdo, sub + bytes_sent, bytes_this_packet);
+            bytes_sent += bytes_this_packet;
           /* test if full sector */
-            i += 20 + header_size + stuffing + svcd;
-            if (i != secsize)
+            bytes_this_packet += 20 + header_size + stuffing + svcd_adjust;
+            if (bytes_this_packet != secsize)
               {
                 unsigned short bs;
               /* if sector not full, write padding? */
@@ -758,7 +806,7 @@ static void mux(int eoinput)
                 c = htonl(0x100 + MPID_PAD); /* really just padding this time */
                 swrite(fdo, &c, 4);
               /* calculate number of padding bytes */
-                b = secsize - i - 6;
+                b = secsize - bytes_this_packet - 6;
                 if (debug > 4)
                   {
                     fprintf(stderr, "INFO: Padding, b: %d\n", b);
@@ -771,7 +819,7 @@ static void mux(int eoinput)
                 for (q = 0; q < b; q++)
                     swrite(fdo, &c, 1);
               } /*if*/
-          } /* end while bytes_send ! sub_size */
+          } /* end while bytes_sent ! sub_size */
 
         if (debug > 0)
           {
@@ -785,40 +833,82 @@ static void mux(int eoinput)
           } /*if*/
         freestinfo(cursti);
       } /*while*/
-  } /*mux*/
+  } /*muxnext*/
+
+static void textsub_statistics()
+  {
+    fprintf(stderr, "\nINFO: Text Subtitle Statistics:\n");
+    fprintf(stderr, "INFO: - Processed %d subtitles.\n", numspus);
+    fprintf(stderr, "INFO: - The longest display line had %d characters.\n", sub_max_chars - 1);
+    fprintf(stderr, "INFO: - The maximum number of displayed lines was %d.\n", sub_max_lines);
+    fprintf(stderr, "INFO: - The normal display height of the font %s was %d.\n", sub_font, sub_max_font_height);
+    fprintf(stderr, "INFO: - The bottom display height of the font %s was %d.\n", sub_font, sub_max_bottom_font_height);
+    fprintf(stderr, "INFO: - The biggest subtitle box had %d bytes.\n", max_sub_size);
+  } /*textsub_statistics*/
 
 int main(int argc,char **argv)
 {
     int fdi;
-    unsigned int c, ch, a, frame;
-    unsigned short int b, vss;
-    unsigned char psbuf[psbufs], ncnt;
+    unsigned int c, ch, a;
+    unsigned short int b;
+    unsigned char psbuf[psbufs];
     int optch;
+#ifdef HAVE_GETOPT_LONG
+    const static struct option longopts[]={
+        {"nodvdauthor-data", 0, 0, 1},
+        {"nomux", 0, 0, 2},
+        {0, 0, 0, 0}
+    };
+#define GETOPTFUNC(x,y,z) getopt_long(x,y,z,longopts,NULL)
+#else
+#define GETOPTFUNC(x,y,z) getopt(x,y,z)
+#endif
 
+    default_video_format = get_video_format();
+    init_locale();
     newsti = 0;
-    mode = 0; /* default DVD */
+    mode = DVD_SUB; /* default */
     sub = malloc(SUB_BUFFER_MAX + SUB_BUFFER_HEADROOM);
     if (!sub)
       {
-        fprintf(stderr, "ERR: Could not allocate space for sub, aborting.\n");
+        fprintf(stderr, "ERR:  Could not allocate space for sub, aborting.\n");
         exit(1);
       } /*if*/
 //fprintf(stderr, "malloc sub=%p\n", sub);
     if (!(cbuf = malloc(65536)))
       {
-        fprintf(stderr, "ERR: Could not allocate space for sub buffer, aborting.\n");
+        fprintf(stderr, "ERR:  Could not allocate space for sub buffer, aborting.\n");
         exit(1);
       } /*if*/
     image_init();
     fputs(PACKAGE_HEADER("spumux"), stderr);
-    gts = 0;
-    nextgts = 0;
+    if (default_video_format != VF_NONE)
+      {
+        fprintf
+          (
+            stderr,
+            "INFO: default video format is %s\n",
+            default_video_format == VF_PAL ? "PAL" : "NTSC"
+          );
+      }
+    else
+      {
+#if defined(DEFAULT_VIDEO_FORMAT)
+#    if DEFAULT_VIDEO_FORMAT == 1
+        fprintf(stderr, "INFO: default video format is NTSC\n");
+        default_video_format = VF_NTSC;
+#    elif DEFAULT_VIDEO_FORMAT == 2
+        fprintf(stderr, "INFO: default video format is PAL\n");
+        default_video_format = VF_PAL;
+#    endif
+#else
+        fprintf(stderr, "INFO: no default video format, must explicitly specify NTSC or PAL\n");
+#endif
+      } /*if*/
     tofs = -1;
-    progr = 0;
     debug = 0;
-    ncnt = 0;
-    substr = 0;
-    while (-1 != (optch = getopt(argc, argv, "hm:s:v:P")))
+    substr = 0; /* default */
+    while (-1 != (optch = GETOPTFUNC(argc, argv, "hm:s:v:P")))
       {
         switch (optch)
           {
@@ -841,113 +931,131 @@ int main(int argc,char **argv)
             break;
 
             default:
-                fprintf(stderr,"ERR: Mode must be one of dvd, svcd, or cvd\n");
+                fprintf(stderr,"ERR:  Mode must be one of dvd, svcd, or cvd\n");
                 usage();
               } /*switch*/
         break;
         case 's':
             substr = strtounsigned(optarg, "substream id");
+            if (substr > 31)
+              {
+                fprintf(stderr, "ERR:  Invalid stream ID, must be in 0 .. 31\n");
+                exit(1);
+              } /*if*/
         break;
         case 'v':
             debug = strtounsigned(optarg, "verbosity");
         break;
         case 'P':
-            progr = 1;
+            show_progress = true;
         break;
         case 'h':
             usage();
         break;
+        case 1: /* --nodvdauthor-data */
+            dodvdauthor_data = false;
+        break;
+        case 2: /* --nomux */
+            domux = false;
+        break;
         default:
             fprintf(stderr, "WARN: Getopt returned %d\n",optch);
             usage();
         break;
           } /*switch*/
       } /*while*/
+    if (argc - optind != 1)
+      {
+        fprintf(stderr, "WARN: Only one argument expected\n");
+        usage();
+      } /*if*/
 
     switch(mode)
       {
     case DVD_SUB:
     default:
-        svcd = 0;
+        svcd_adjust = 0;
         substr += DVD_SUB_CHANNEL;
         muxrate = 10080 * 10 / 4; // 0x1131; // 10080 kbps
         secsize = 2048;
     break;
     case CVD_SUB:
-        svcd = 0;
+        svcd_adjust = 0;
         substr += CVD_SUB_CHANNEL;
         muxrate = 1040 * 10/4; //0x0a28; // 1040 kbps
         secsize = 2324;
     break;
     case SVCD_SUB:
-        svcd = 4;
+        svcd_adjust = 4;
         // svcd substream identification works differently...
         // substr += SVCD_SUB_CHANNEL;
         muxrate = 1760 * 10 / 4; //0x1131; // 1760 kbps
         secsize = 2324;
     break;
       } /*switch*/
-    if (argc - optind != 1)
+    fdi = 0; /* stdin */
+    fdo = 1; /* stdout */
+    if (domux)
       {
-        fprintf(stderr, "WARN: Only one argument expected\n");
-        usage();
+        win32_setmode(fdi,O_BINARY);
       } /*if*/
-    fdi = 0;
-    fdo = 1;
-    win32_setmode(fdi,O_BINARY);
     win32_setmode(fdo,O_BINARY);
     if (spumux_parse(argv[optind]))
         return -1;
     if (tofs >= 0 && debug > 0)
         fprintf(stderr, "INFO: Subtitles offset by %fs\n", (double)tofs / 90000);
+    if (have_textsub)
+      {
+        vo_init_osd();
+      } /*if*/
     spuindex = 0;
-    skip = 0;
+    nr_subtitles_skipped = 0;
     if (!(sector = malloc(secsize)))
       {
-        fprintf(stderr, "ERR: Could not allocate space for sector buffer, aborting.\n");
+        fprintf(stderr, "ERR:  Could not allocate space for sector buffer, aborting.\n");
         exit(1);
       } /*if*/
+    memset(substream_present, false, sizeof substream_present);
 
     newsti = getnextsub();
     max_sub_size = 0;
-    header_size = 12;
-    vss = 0;
-    frame = 0;
+    header_size = 12; /* first PES header extension will have PTS data and a PES extension */
     lps = 0;
-    gts = 0;
+    lastgts = 0;
+    nextgts = 0;
     subno = -1;
-    while (1)
+    while (domux)
       {
-        mux(0);
+        muxnext(false);
         if (sread(fdi, &c, 4) != 4)
             goto eoi;
         ch = ntohl(c); /* header ID */
         if (ch == 0x100 + MPID_PACK)
           {
 l_01ba:
-            if (progr) /* show progress */
+            if (show_progress) /* show progress */
               {
                 if (lps % 1024 * 1024 * 10 < secsize)
                     fprintf(stderr, "INFO: %" PRIu64 " bytes of data written\r", lps);
               } /*if*/
             if (debug > 5)
                 fprintf(stderr, "INFO: pack_start_code\n");
-            if (sread( fdi, psbuf, psbufs) != psbufs)
+            if (sread(fdi, psbuf, psbufs) != psbufs)
                 break;
-            gts = getgts(psbuf);
-            if (gts != -1)
+            lastgts = getgts(psbuf);
+            if (lastgts != -1)
               {
-                mux(0);
-                fixgts(&gts, &nextgts);
+                muxnext(false);
+                fixgts(&lastgts, &nextgts);
                 muxrate = getmuxr(psbuf);
               }
             else
               {
                 if (debug >- 1)
                     fprintf(stderr, "WARN: Incorrect pack header\n");
-                gts = nextgts;
+                lastgts = nextgts;
               } /*if*/
-            mkpackh(gts, muxrate, 0);
+            mkpackh(lastgts, muxrate, 0);
             swrite(fdo, &c, 4);
             swrite(fdo, header, psbufs);
           }
@@ -960,6 +1068,62 @@ l_01ba:
             b = ntohs(b);
             if (sread(fdi, cbuf, b) != b) /* packet contents */
                 break;
+            if (ch == 0x100 + MPID_PRIVATE1)
+              {
+                const int dptr = cbuf[2] /* PES header data length */ + 3 /* offset to packet data */;
+                const int substreamid =
+                    mode == SVCD_SUB ?
+                        cbuf[dptr] == SVCD_SUB_CHANNEL ?
+                            cbuf[dptr + 1]
+                        :
+                            -1 /*?*/
+                    : (cbuf[dptr] & 0xE0) != 0x80 ? /* not DVD-Video-specific audio */
+                        cbuf[dptr]
+                    :
+                        -1;
+                if (substreamid >= 0)
+                  {
+                    if (substreamid == substr)
+                      {
+                        fprintf(stderr, "ERR:  duplicate substream ID 0x%02x\n", substr);
+                        exit(1);
+                      } /*if*/
+                    if (!substream_present[substreamid])
+                      {
+                        const char * modestr;
+                        int streamid;
+                        if (mode == DVD_SUB && substreamid >= DVD_SUB_CHANNEL)
+                          {
+                            modestr = "DVD-Video";
+                            streamid = substreamid - DVD_SUB_CHANNEL;
+                          }
+                        else if (mode == SVCD_SUB)
+                          {
+                            modestr = "SVCD";
+                            streamid = substreamid;
+                          }
+                        else if (mode == CVD_SUB)
+                          {
+                            modestr = "CVD";
+                            streamid = substreamid;
+                          }
+                        else
+                          {
+                            modestr= "?";
+                            streamid = substreamid;
+                          } /*if*/
+                        fprintf
+                          (
+                            stderr,
+                            "INFO: found existing substream ID 0x%02x (%s %d)\n",
+                            substreamid,
+                            modestr,
+                            streamid
+                          );
+                        substream_present[substreamid] = true;
+                      } /*if*/
+                  } /*if*/
+              } /*if*/
             swrite(fdo, cbuf, b);
             if (ch == 0x100 + MPID_VIDEO_FIRST && tofs == -1)
               { /* video stream (DVD only allows one) */
@@ -1003,22 +1167,24 @@ l_01ba:
             goto l_01ba;
           } /*if*/
       } /*while*/
-
  eoi:
-    mux(1); // end of input
+    muxnext(true); // end of input
 /*    fprintf(stderr, "max_sub_size=%d\n", max_sub_size); */
     if (subno != 0xffff)
       {
         fprintf(stderr,
             "INFO: %d subtitles added, %d subtitles skipped, stream: %d, offset: %.2f\n",
-            subno + 1, skip, substr, (double)tofs / 90000);
+            subno + 1, nr_subtitles_skipped, substr, (double)tofs / 90000);
       }
     else
       {
         fprintf(stderr, "WARN: no subtitles added\n");
       } /*if*/
-    textsub_statistics();
-    textsub_finish();
+    if (have_textsub)
+      {
+        textsub_statistics();
+        vo_finish_osd();
+      } /*if*/
     image_shutdown();
     return 0;
 } /* end function main */
